import imp
import os
import py_compile
import shutil
import tempfile
import unittest

from test import test_support

class PyCompileTests(unittest.TestCase):

    def setUp(self):
        self.directory = tempfile.mkdtemp()
        self.source_path = os.path.join(self.directory, '_test.py')
        self.pyc_path = self.source_path + 'c'
        self.cwd_drive = os.path.splitdrive(os.getcwd())[0]
        # In these tests we compute relative paths.  When using Windows, the
        # current working directory path and the 'self.source_path' might be
        # on different drives.  Therefore we need to switch to the drive where
        # the temporary source file lives.
        drive = os.path.splitdrive(self.source_path)[0]
        if drive:
            os.chdir(drive)

        with open(self.source_path, 'w') as file:
            file.write('x = 123\n')

    def tearDown(self):
        shutil.rmtree(self.directory)
        if self.cwd_drive:
            os.chdir(self.cwd_drive)

    def test_absolute_path(self):
        py_compile.compile(self.source_path, self.pyc_path)
        self.assertTrue(os.path.exists(self.pyc_path))

    def test_cwd(self):
        cwd = os.getcwd()
        os.chdir(self.directory)
        py_compile.compile(os.path.basename(self.source_path),
                           os.path.basename(self.pyc_path))
        os.chdir(cwd)
        self.assertTrue(os.path.exists(self.pyc_path))

    def test_relative_path(self):
        py_compile.compile(os.path.relpath(self.source_path),
                           os.path.relpath(self.pyc_path))
        self.assertTrue(os.path.exists(self.pyc_path))

def test_main():
    test_support.run_unittest(PyCompileTests)

if __name__ == "__main__":
    test_main()



"""Convert a NT pathname to a file URL and vice versa."""

def url2pathname(url):
    """OS-specific conversion from a relative URL of the 'file' scheme
    to a file system path; not recommended for general use."""
    # e.g.
    # ///C|/foo/bar/spam.foo
    # becomes
    # C:\foo\bar\spam.foo
    import string, urllib
    # Windows itself uses ":" even in URLs.
    url = url.replace(':', '|')
    if not '|' in url:
        # No drive specifier, just convert slashes
        if url[:4] == '////':
            # path is something like ////host/path/on/remote/host
            # convert this to \\host\path\on\remote\host
            # (notice halving of slashes at the start of the path)
            url = url[2:]
        components = url.split('/')
        # make sure not to convert quoted slashes :-)
        return urllib.unquote('\\'.join(components))
    comp = url.split('|')
    if len(comp) != 2 or comp[0][-1] not in string.ascii_letters:
        error = 'Bad URL: ' + url
        raise IOError, error
    drive = comp[0][-1].upper()
    path = drive + ':'
    components = comp[1].split('/')
    for comp in components:
        if comp:
            path = path + '\\' + urllib.unquote(comp)
    # Issue #11474: url like '/C|/' should convert into 'C:\\'
    if path.endswith(':') and url.endswith('/'):
        path += '\\'
    return path

def pathname2url(p):
    """OS-specific conversion from a file system path to a relative URL
    of the 'file' scheme; not recommended for general use."""
    # e.g.
    # C:\foo\bar\spam.foo
    # becomes
    # ///C|/foo/bar/spam.foo
    import urllib
    if not ':' in p:
        # No drive specifier, just convert slashes and quote the name
        if p[:2] == '\\\\':
        # path is something like \\host\path\on\remote\host
        # convert this to ////host/path/on/remote/host
        # (notice doubling of slashes at the start of the path)
            p = '\\\\' + p
        components = p.split('\\')
        return urllib.quote('/'.join(components))
    comp = p.split(':')
    if len(comp) != 2 or len(comp[0]) > 1:
        error = 'Bad path: ' + p
        raise IOError, error

    drive = urllib.quote(comp[0].upper())
    components = comp[1].split('\\')
    path = '///' + drive + ':'
    for comp in components:
        if comp:
            path = path + '/' + urllib.quote(comp)
    return path



import os
import fnmatch
import re  # for htest
import sys
from Tkinter import StringVar, BooleanVar, Checkbutton  # for GrepDialog
from Tkinter import Tk, Text, Button, SEL, END  # for htest
from idlelib import SearchEngine
import itertools
from idlelib.SearchDialogBase import SearchDialogBase
# Importing OutputWindow fails due to import loop
# EditorWindow -> GrepDialop -> OutputWindow -> EditorWindow

def grep(text, io=None, flist=None):
    root = text._root()
    engine = SearchEngine.get(root)
    if not hasattr(engine, "_grepdialog"):
        engine._grepdialog = GrepDialog(root, engine, flist)
    dialog = engine._grepdialog
    searchphrase = text.get("sel.first", "sel.last")
    dialog.open(text, searchphrase, io)

class GrepDialog(SearchDialogBase):

    title = "Find in Files Dialog"
    icon = "Grep"
    needwrapbutton = 0

    def __init__(self, root, engine, flist):
        SearchDialogBase.__init__(self, root, engine)
        self.flist = flist
        self.globvar = StringVar(root)
        self.recvar = BooleanVar(root)

    def open(self, text, searchphrase, io=None):
        SearchDialogBase.open(self, text, searchphrase)
        if io:
            path = io.filename or ""
        else:
            path = ""
        dir, base = os.path.split(path)
        head, tail = os.path.splitext(base)
        if not tail:
            tail = ".py"
        self.globvar.set(os.path.join(dir, "*" + tail))

    def create_entries(self):
        SearchDialogBase.create_entries(self)
        self.globent = self.make_entry("In files:", self.globvar)[0]

    def create_other_buttons(self):
        f = self.make_frame()[0]

        btn = Checkbutton(f, anchor="w",
                variable=self.recvar,
                text="Recurse down subdirectories")
        btn.pack(side="top", fill="both")
        btn.select()

    def create_command_buttons(self):
        SearchDialogBase.create_command_buttons(self)
        self.make_button("Search Files", self.default_command, 1)

    def default_command(self, event=None):
        prog = self.engine.getprog()
        if not prog:
            return
        path = self.globvar.get()
        if not path:
            self.top.bell()
            return
        from idlelib.OutputWindow import OutputWindow  # leave here!
        save = sys.stdout
        try:
            sys.stdout = OutputWindow(self.flist)
            self.grep_it(prog, path)
        finally:
            sys.stdout = save

    def grep_it(self, prog, path):
        dir, base = os.path.split(path)
        list = self.findfiles(dir, base, self.recvar.get())
        list.sort()
        self.close()
        pat = self.engine.getpat()
        print("Searching %r in %s ..." % (pat, path))
        hits = 0
        try:
            for fn in list:
                try:
                    with open(fn) as f:
                        for lineno, line in enumerate(f, 1):
                            if line[-1:] == '\n':
                                line = line[:-1]
                            if prog.search(line):
                                sys.stdout.write("%s: %s: %s\n" %
                                                 (fn, lineno, line))
                                hits += 1
                except IOError as msg:
                    print(msg)
            print(("Hits found: %s\n"
                  "(Hint: right-click to open locations.)"
                  % hits) if hits else "No hits.")
        except AttributeError:
            # Tk window has been closed, OutputWindow.text = None,
            # so in OW.write, OW.text.insert fails.
            pass

    def findfiles(self, dir, base, rec):
        try:
            names = os.listdir(dir or os.curdir)
        except os.error as msg:
            print(msg)
            return []
        list = []
        subdirs = []
        for name in names:
            fn = os.path.join(dir, name)
            if os.path.isdir(fn):
                subdirs.append(fn)
            else:
                if fnmatch.fnmatch(name, base):
                    list.append(fn)
        if rec:
            for subdir in subdirs:
                list.extend(self.findfiles(subdir, base, rec))
        return list

    def close(self, event=None):
        if self.top:
            self.top.grab_release()
            self.top.withdraw()


def _grep_dialog(parent):  # htest #
    from idlelib.PyShell import PyShellFileList
    root = Tk()
    root.title("Test GrepDialog")
    width, height, x, y = list(map(int, re.split('[x+]', parent.geometry())))
    root.geometry("+%d+%d"%(x, y + 150))

    flist = PyShellFileList(root)
    text = Text(root, height=5)
    text.pack()

    def show_grep_dialog():
        text.tag_add(SEL, "1.0", END)
        grep(text, flist=flist)
        text.tag_remove(SEL, "1.0", END)

    button = Button(root, text="Show GrepDialog", command=show_grep_dialog)
    button.pack()
    root.mainloop()

if __name__ == "__main__":
    import unittest
    unittest.main('idlelib.idle_test.test_grep', verbosity=2, exit=False)

    from idlelib.idle_test.htest import run
    run(_grep_dialog)



# Copyright 2007 Google, Inc. All Rights Reserved.
# Licensed to PSF under a Contributor Agreement.

"""Fixer for callable().

This converts callable(obj) into isinstance(obj, collections.Callable), adding a
collections import if needed."""

# Local imports
from lib2to3 import fixer_base
from lib2to3.fixer_util import Call, Name, String, Attr, touch_import

class FixCallable(fixer_base.BaseFix):
    BM_compatible = True

    order = "pre"

    # Ignore callable(*args) or use of keywords.
    # Either could be a hint that the builtin callable() is not being used.
    PATTERN = """
    power< 'callable'
           trailer< lpar='('
                    ( not(arglist | argument<any '=' any>) func=any
                      | func=arglist<(not argument<any '=' any>) any ','> )
                    rpar=')' >
           after=any*
    >
    """

    def transform(self, node, results):
        func = results['func']

        touch_import(None, u'collections', node=node)

        args = [func.clone(), String(u', ')]
        args.extend(Attr(Name(u'collections'), Name(u'Callable')))
        return Call(Name(u'isinstance'), args, prefix=node.prefix)




#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# See the License for the specific language governing permissions
# and limitations under the License.
#
#
# Copyright 2011 Grigale Ltd. All rigths reserved.
# Use is sujbect to license terms.
#

__version__ = '1'
__author__ = 'Cyril Plisko <cyril.plisko@grigale.com>'

from .kstat import Kstat

__all__ = ['Kstat']



import sys
import os
import unittest
import platform
import subprocess

from test import test_support

class PlatformTest(unittest.TestCase):
    def test_architecture(self):
        res = platform.architecture()

    if hasattr(os, "symlink"):
        def test_architecture_via_symlink(self): # issue3762
            def get(python):
                cmd = [python, '-c',
                    'import platform; print platform.architecture()']
                p = subprocess.Popen(cmd, stdout=subprocess.PIPE)
                return p.communicate()
            real = os.path.realpath(sys.executable)
            link = os.path.abspath(test_support.TESTFN)
            os.symlink(real, link)
            try:
                self.assertEqual(get(real), get(link))
            finally:
                os.remove(link)

    def test_platform(self):
        for aliased in (False, True):
            for terse in (False, True):
                res = platform.platform(aliased, terse)

    def test_system(self):
        res = platform.system()

    def test_node(self):
        res = platform.node()

    def test_release(self):
        res = platform.release()

    def test_version(self):
        res = platform.version()

    def test_machine(self):
        res = platform.machine()

    def test_processor(self):
        res = platform.processor()

    def setUp(self):
        self.save_version = sys.version
        self.save_subversion = sys.subversion
        self.save_platform = sys.platform

    def tearDown(self):
        sys.version = self.save_version
        sys.subversion = self.save_subversion
        sys.platform = self.save_platform

    def test_sys_version(self):
        # Old test.
        for input, output in (
            ('2.4.3 (#1, Jun 21 2006, 13:54:21) \n[GCC 3.3.4 (pre 3.3.5 20040809)]',
             ('CPython', '2.4.3', '', '', '1', 'Jun 21 2006 13:54:21', 'GCC 3.3.4 (pre 3.3.5 20040809)')),
            ('IronPython 1.0.60816 on .NET 2.0.50727.42',
             ('IronPython', '1.0.60816', '', '', '', '', '.NET 2.0.50727.42')),
            ('IronPython 1.0 (1.0.61005.1977) on .NET 2.0.50727.42',
             ('IronPython', '1.0.0', '', '', '', '', '.NET 2.0.50727.42')),
            ):
            # branch and revision are not "parsed", but fetched
            # from sys.subversion.  Ignore them
            (name, version, branch, revision, buildno, builddate, compiler) \
                   = platform._sys_version(input)
            self.assertEqual(
                (name, version, '', '', buildno, builddate, compiler), output)

        # Tests for python_implementation(), python_version(), python_branch(),
        # python_revision(), python_build(), and python_compiler().
        sys_versions = {
            ("2.6.1 (r261:67515, Dec  6 2008, 15:26:00) \n[GCC 4.0.1 (Apple Computer, Inc. build 5370)]",
             ('CPython', 'tags/r261', '67515'), self.save_platform)
            :
                ("CPython", "2.6.1", "tags/r261", "67515",
                 ('r261:67515', 'Dec  6 2008 15:26:00'),
                 'GCC 4.0.1 (Apple Computer, Inc. build 5370)'),

            ("IronPython 2.0 (2.0.0.0) on .NET 2.0.50727.3053", None, "cli")
            :
                ("IronPython", "2.0.0", "", "", ("", ""),
                 ".NET 2.0.50727.3053"),

            ("2.6.1 (IronPython 2.6.1 (2.6.10920.0) on .NET 2.0.50727.1433)", None, "cli")
            :
                ("IronPython", "2.6.1", "", "", ("", ""),
                 ".NET 2.0.50727.1433"),

            ("2.7.4 (IronPython 2.7.4 (2.7.0.40) on Mono 4.0.30319.1 (32-bit))", None, "cli")
            :
                ("IronPython", "2.7.4", "", "", ("", ""),
                 "Mono 4.0.30319.1 (32-bit)"),

            ("2.5 (trunk:6107, Mar 26 2009, 13:02:18) \n[Java HotSpot(TM) Client VM (\"Apple Computer, Inc.\")]",
            ('Jython', 'trunk', '6107'), "java1.5.0_16")
            :
                ("Jython", "2.5.0", "trunk", "6107",
                 ('trunk:6107', 'Mar 26 2009'), "java1.5.0_16"),

            ("2.5.2 (63378, Mar 26 2009, 18:03:29)\n[PyPy 1.0.0]",
             ('PyPy', 'trunk', '63378'), self.save_platform)
            :
                ("PyPy", "2.5.2", "trunk", "63378", ('63378', 'Mar 26 2009'),
                 "")
            }
        for (version_tag, subversion, sys_platform), info in \
                sys_versions.iteritems():
            sys.version = version_tag
            if subversion is None:
                if hasattr(sys, "subversion"):
                    del sys.subversion
            else:
                sys.subversion = subversion
            if sys_platform is not None:
                sys.platform = sys_platform
            self.assertEqual(platform.python_implementation(), info[0])
            self.assertEqual(platform.python_version(), info[1])
            self.assertEqual(platform.python_branch(), info[2])
            self.assertEqual(platform.python_revision(), info[3])
            self.assertEqual(platform.python_build(), info[4])
            self.assertEqual(platform.python_compiler(), info[5])

    def test_system_alias(self):
        res = platform.system_alias(
            platform.system(),
            platform.release(),
            platform.version(),
        )

    def test_uname(self):
        res = platform.uname()
        self.assertTrue(any(res))

    @unittest.skipUnless(sys.platform.startswith('win'), "windows only test")
    def test_uname_win32_ARCHITEW6432(self):
        # Issue 7860: make sure we get architecture from the correct variable
        # on 64 bit Windows: if PROCESSOR_ARCHITEW6432 exists we should be
        # using it, per
        # http://blogs.msdn.com/david.wang/archive/2006/03/26/HOWTO-Detect-Process-Bitness.aspx
        try:
            with test_support.EnvironmentVarGuard() as environ:
                if 'PROCESSOR_ARCHITEW6432' in environ:
                    del environ['PROCESSOR_ARCHITEW6432']
                environ['PROCESSOR_ARCHITECTURE'] = 'foo'
                platform._uname_cache = None
                system, node, release, version, machine, processor = platform.uname()
                self.assertEqual(machine, 'foo')
                environ['PROCESSOR_ARCHITEW6432'] = 'bar'
                platform._uname_cache = None
                system, node, release, version, machine, processor = platform.uname()
                self.assertEqual(machine, 'bar')
        finally:
            platform._uname_cache = None

    def test_java_ver(self):
        res = platform.java_ver()
        if sys.platform == 'java':
            self.assertTrue(all(res))

    def test_win32_ver(self):
        res = platform.win32_ver()

    def test_mac_ver(self):
        res = platform.mac_ver()

        try:
            import gestalt
        except ImportError:
            have_toolbox_glue = False
        else:
            have_toolbox_glue = True

        if have_toolbox_glue and platform.uname()[0] == 'Darwin':
            # We're on a MacOSX system, check that
            # the right version information is returned
            fd = os.popen('sw_vers', 'r')
            real_ver = None
            for ln in fd:
                if ln.startswith('ProductVersion:'):
                    real_ver = ln.strip().split()[-1]
                    break
            fd.close()
            self.assertFalse(real_ver is None)
            result_list = res[0].split('.')
            expect_list = real_ver.split('.')
            len_diff = len(result_list) - len(expect_list)
            # On Snow Leopard, sw_vers reports 10.6.0 as 10.6
            if len_diff > 0:
                expect_list.extend(['0'] * len_diff)
            self.assertEqual(result_list, expect_list)

            # res[1] claims to contain
            # (version, dev_stage, non_release_version)
            # That information is no longer available
            self.assertEqual(res[1], ('', '', ''))

            if sys.byteorder == 'little':
                self.assertIn(res[2], ('i386', 'x86_64'))
            else:
                self.assertEqual(res[2], 'PowerPC')


    @unittest.skipUnless(sys.platform == 'darwin', "OSX only test")
    def test_mac_ver_with_fork(self):
        # Issue7895: platform.mac_ver() crashes when using fork without exec
        #
        # This test checks that the fix for that issue works.
        #
        pid = os.fork()
        if pid == 0:
            # child
            info = platform.mac_ver()
            os._exit(0)

        else:
            # parent
            cpid, sts = os.waitpid(pid, 0)
            self.assertEqual(cpid, pid)
            self.assertEqual(sts, 0)

    def test_dist(self):
        res = platform.dist()

    def test_libc_ver(self):
        import os
        if os.path.isdir(sys.executable) and \
           os.path.exists(sys.executable+'.exe'):
            # Cygwin horror
            executable = sys.executable + '.exe'
        else:
            executable = sys.executable
        res = platform.libc_ver(executable)

    def test_parse_release_file(self):

        for input, output in (
            # Examples of release file contents:
            ('SuSE Linux 9.3 (x86-64)', ('SuSE Linux ', '9.3', 'x86-64')),
            ('SUSE LINUX 10.1 (X86-64)', ('SUSE LINUX ', '10.1', 'X86-64')),
            ('SUSE LINUX 10.1 (i586)', ('SUSE LINUX ', '10.1', 'i586')),
            ('Fedora Core release 5 (Bordeaux)', ('Fedora Core', '5', 'Bordeaux')),
            ('Red Hat Linux release 8.0 (Psyche)', ('Red Hat Linux', '8.0', 'Psyche')),
            ('Red Hat Linux release 9 (Shrike)', ('Red Hat Linux', '9', 'Shrike')),
            ('Red Hat Enterprise Linux release 4 (Nahant)', ('Red Hat Enterprise Linux', '4', 'Nahant')),
            ('CentOS release 4', ('CentOS', '4', None)),
            ('Rocks release 4.2.1 (Cydonia)', ('Rocks', '4.2.1', 'Cydonia')),
            ('', ('', '', '')), # If there's nothing there.
            ):
            self.assertEqual(platform._parse_release_file(input), output)


def test_main():
    test_support.run_unittest(
        PlatformTest
    )

if __name__ == '__main__':
    test_main()



from Tkinter import *
from idlelib.EditorWindow import EditorWindow
import re
import tkMessageBox
from idlelib import IOBinding

class OutputWindow(EditorWindow):

    """An editor window that can serve as an output file.

    Also the future base class for the Python shell window.
    This class has no input facilities.
    """

    def __init__(self, *args):
        EditorWindow.__init__(self, *args)
        self.text.bind("<<goto-file-line>>", self.goto_file_line)

    # Customize EditorWindow

    def ispythonsource(self, filename):
        # No colorization needed
        return 0

    def short_title(self):
        return "Output"

    def maybesave(self):
        # Override base class method -- don't ask any questions
        if self.get_saved():
            return "yes"
        else:
            return "no"

    # Act as output file

    def write(self, s, tags=(), mark="insert"):
        # Tk assumes that byte strings are Latin-1;
        # we assume that they are in the locale's encoding
        if isinstance(s, str):
            try:
                s = unicode(s, IOBinding.encoding)
            except UnicodeError:
                # some other encoding; let Tcl deal with it
                pass
        self.text.insert(mark, s, tags)
        self.text.see(mark)
        self.text.update()

    def writelines(self, lines):
        for line in lines:
            self.write(line)

    def flush(self):
        pass

    # Our own right-button menu

    rmenu_specs = [
        ("Cut", "<<cut>>", "rmenu_check_cut"),
        ("Copy", "<<copy>>", "rmenu_check_copy"),
        ("Paste", "<<paste>>", "rmenu_check_paste"),
        (None, None, None),
        ("Go to file/line", "<<goto-file-line>>", None),
    ]

    file_line_pats = [
        # order of patterns matters
        r'file "([^"]*)", line (\d+)',
        r'([^\s]+)\((\d+)\)',
        r'^(\s*\S.*?):\s*(\d+):',  # Win filename, maybe starting with spaces
        r'([^\s]+):\s*(\d+):',     # filename or path, ltrim
        r'^\s*(\S.*?):\s*(\d+):',  # Win abs path with embedded spaces, ltrim
    ]

    file_line_progs = None

    def goto_file_line(self, event=None):
        if self.file_line_progs is None:
            l = []
            for pat in self.file_line_pats:
                l.append(re.compile(pat, re.IGNORECASE))
            self.file_line_progs = l
        # x, y = self.event.x, self.event.y
        # self.text.mark_set("insert", "@%d,%d" % (x, y))
        line = self.text.get("insert linestart", "insert lineend")
        result = self._file_line_helper(line)
        if not result:
            # Try the previous line.  This is handy e.g. in tracebacks,
            # where you tend to right-click on the displayed source line
            line = self.text.get("insert -1line linestart",
                                 "insert -1line lineend")
            result = self._file_line_helper(line)
            if not result:
                tkMessageBox.showerror(
                    "No special line",
                    "The line you point at doesn't look like "
                    "a valid file name followed by a line number.",
                    master=self.text)
                return
        filename, lineno = result
        edit = self.flist.open(filename)
        edit.gotoline(lineno)

    def _file_line_helper(self, line):
        for prog in self.file_line_progs:
            match = prog.search(line)
            if match:
                filename, lineno = match.group(1, 2)
                try:
                    f = open(filename, "r")
                    f.close()
                    break
                except IOError:
                    continue
        else:
            return None
        try:
            return filename, int(lineno)
        except TypeError:
            return None

# These classes are currently not used but might come in handy

class OnDemandOutputWindow:

    tagdefs = {
        # XXX Should use IdlePrefs.ColorPrefs
        "stdout":  {"foreground": "blue"},
        "stderr":  {"foreground": "#007700"},
    }

    def __init__(self, flist):
        self.flist = flist
        self.owin = None

    def write(self, s, tags, mark):
        if not self.owin:
            self.setup()
        self.owin.write(s, tags, mark)

    def setup(self):
        self.owin = owin = OutputWindow(self.flist)
        text = owin.text
        for tag, cnf in self.tagdefs.items():
            if cnf:
                text.tag_configure(tag, **cnf)
        text.tag_raise('sel')
        self.write = self.owin.write





NET_SECURE_HIDE   = 0x01
NET_SECURE_CYPHER = 0x02
NET_RESERVED1     = 0x04
NET_RESERVED2     = 0x08
NET_RESERVED3     = 0x10
NET_RESERVED4     = 0x20
NET_RESERVED5     = 0x40
NET_RESERVED6     = 0x80

NET_KEY_EXTENSION = 0xfd
NET_TIME_SYNC     = 0xfe
NET_KEY_RESET     = 0xff

NETSRC_AGENT_JAVA_EMBED   = 1
NETSRC_AGENT_JAVA_WATCHER = 2
NETSRC_SERVER_YARD        = 3
NETSRC_SERVER_PROXY       = 4

TCP_NONE = 0x00
TCP_OK   = 0x01

PARAM_KEEPALIVE = 1
PARAM_AGENT_BOOT_ENV = 2
PARAM_CONFIGURE_UPDATE = 3
PARAM_CONFIGURE_GET = 4
PARAM_COMPO_VERSIONS = 5
PARAM_THREAD_LIST = 7
PARAM_THREAD_DETAIL = 8
PARAM_GET_ACTIVE_STACK = 9
PARAM_HEAP_HISTO = 10
PARAM_LOADED_CLASS_LIST = 11
PARAM_GET_ENV = 12
PARAM_SYSTEM_GC = 13
PARAM_SET_CONFIG = 14
PARAM_OPEN_SOCKET_LIST = 15
PARAM_LOADED_CLASS_DETAIL = 16
PARAM_LOADED_CLASS_REDEFINE = 17
PARAM_GET_CONFIG = 200

PARAM_AGENT_JAR_LIST = 18
PARAM_AGENT_JAR_SAVE = 19
PARAM_AGENT_JAR_DELETE = 20

PARAM_RESET_STRING_SENT_MARK = 21

PARAM_AGENT_LOG_LIST = 22
PARAM_AGENT_LOG_READ = 23
PARAM_THREAD_CONTROL = 24

PARAM_GET_TOPOLOGY = 27

PARAM_NET_STAT = 28
PARAM_PS_EF = 29
PARAM_DOCKER_LIST = 30

PARAM_REMOTE_CMD= 33
PARAM_INFRA_HOSTINFO        = 49

# Script Manager
PARAM_SCRIPT_MANAGER_REQUEST    = 701
PARAM_SCRIPT_MANAGER_EXEC       = 702
PARAM_SCRIPT_MANAGER_STATUS     = 703
PARAM_SCRIPT_MANAGER_FILEREAD   = 704
PARAM_SCRIPT_MANAGER_FILESAVE   = 705
PARAM_SCRIPT_MANAGER_FILELIST   = 706
PARAM_SCRIPT_MANAGER_FILEDELETE = 707
# Agent Update
PARAM_SCRIPT_MANAGER_PATCH        = 708
PARAM_SCRIPT_MANAGER_PATCH_STATUS = 709

# Infra config sync
PARAM_INFRA_AGENT_CONF_REQUEST = 800

def GetSecureMask(code) :
    if code < 0 :
        return 0

    return (code & NET_SECURE_HIDE) | (code & NET_SECURE_CYPHER)


import time
def startNet():
    import receiver
    import tcpsession
    while True:
        session = tcpsession.GetTcpSession()
        if session and session.isOpen() and receiver.getReceiver().isTimeSynced():
            break
        time.sleep(1)


import os
import sys
import json
import time
import signal
import subprocess
import threading
import csv
import base64
from datetime import datetime
from StringIO import StringIO

try:
    from croniter import croniter
except ImportError:
    croniter = None


class SimpleCroniter:
    def __init__(self, spec, base_time=None):
        self.spec = spec
        self.base_time = base_time or datetime.now()
        parts = spec.split()
        if len(parts) != 5:
            raise ValueError("Invalid cron spec (must have 5 fields): {}".format(spec))
        self.minute = self._parse_field(parts[0], 0, 59)
        self.hour = self._parse_field(parts[1], 0, 23)
        self.day = self._parse_field(parts[2], 1, 31)
        self.month = self._parse_field(parts[3], 1, 12)
        self.weekday = self._parse_field(parts[4], 0, 6)

    def _parse_field(self, field, min_val, max_val):
        if field == '*':
            return set(range(min_val, max_val + 1))

        result = set()
        for part in field.split(','):
            part = part.strip()
            if '/' in part:
                base, step = part.split('/', 1)
                step = int(step)
                if base == '*':
                    result.update(range(min_val, max_val + 1, step))
                elif '-' in base:
                    start, end = map(int, base.split('-', 1))
                    result.update(range(start, end + 1, step))
                else:
                    start = int(base)
                    result.update(range(start, max_val + 1, step))
            elif '-' in part:
                start, end = map(int, part.split('-', 1))
                result.update(range(start, end + 1))
            else:
                result.add(int(part))
        return result

    def _matches(self, dt):
        if dt.minute not in self.minute:
            return False
        if dt.hour not in self.hour:
            return False
        if dt.day not in self.day:
            return False
        if dt.month not in self.month:
            return False
        if dt.weekday() not in self.weekday:
            return False
        return True

    def get_next(self, return_type=None):
        from datetime import timedelta

        current = self.base_time + timedelta(minutes=1)
        current = current.replace(second=0, microsecond=0)

        max_iterations = 366 * 24 * 60 * 2
        for _ in range(max_iterations):
            if self._matches(current):
                self.base_time = current
                return current
            current += timedelta(minutes=1)

        raise ValueError("No valid next run time found within 2 years for spec: {}".format(self.spec))


def get_croniter(spec, base_time=None):
    if croniter is not None:
        return croniter(spec, base_time)
    return SimpleCroniter(spec, base_time)

import whatap.agent.conf.configure as config
import whatap.agent.secure.security_master as secure
import whatap.agent.data.data_pack as data
import whatap.pack.tagcount_pack as tagcount_pack
import whatap.pack.param_pack as param_pack
import whatap.util.logging_util as logging_util
from whatap.util.date_util import DateUtil as dateutil
from whatap.util.hash_util import HashUtil
import whatap.net as net
from whatap.value.text_value import TextValue
from whatap.value.decimal_value import DecimalValue
from whatap.value.float_value import FloatValue
from whatap.value.list_value import ListValue
from whatap.value.map_value import MapValue

# Constants
SCHEDULE_TYPE_CRON = "cron"
SCHEDULE_TYPE_INTERVAL = "interval"

STATUS_INIT = "init"
STATUS_RUNNING = "running"
STATUS_SUCCESS = "success"
STATUS_ERROR = "error"

CONFIG_POLL_INTERVAL = 5  # seconds


class JobStatus(object):
    def __init__(self, job_id):
        self.id = job_id
        self.schedule = ""
        self.schedule_type = ""
        self.last_execute_start_time = 0
        self.last_execute_status = STATUS_INIT
        self.last_execute_duration = 0
        self.error_message = ""


class Script(object):
    def __init__(self, data=None):
        data = data or {}
        self.file_name = data.get('fileName', '')
        self.type = data.get('type', 0)  # 0: csv, 1: csv, 2: json
        self.delimiter = data.get('delimiter', '|')
        self.content = data.get('content', '')


class ScriptConfig(object):
    def __init__(self, data=None):
        data = data or {}
        self.file_name = data.get('fileName', '')
        self.content = data.get('content', '')


class Trigger(object):
    def __init__(self, data=None):
        data = data or {}
        self.cron = data.get('cron')
        self.interval = data.get('interval')


class Column(object):
    def __init__(self, data=None):
        data = data or {}
        self.type = data.get('type', 'string')
        self.required = data.get('required', True)


class OutputTagCount(object):
    def __init__(self, data=None):
        data = data or {}
        self.category_name = data.get('categoryName', '')
        self.column_map = {}
        for k, v in data.get('columnMap', {}).items():
            self.column_map[k] = Column(v)
        self.fold_rec_type = data.get('foldRecType', 0)


class OutputKvTable(object):
    def __init__(self, data=None):
        data = data or {}
        self.category_name = data.get('categoryName', '')
        self.column_map = {}
        for k, v in data.get('columnMap', {}).items():
            self.column_map[k] = Column(v)


class Output(object):
    def __init__(self, data=None):
        data = data or {}
        self.tag_count = None
        self.kv_table = None

        if data.get('tagCount'):
            self.tag_count = OutputTagCount(data['tagCount'])
        if data.get('kvTable'):
            self.kv_table = OutputKvTable(data['kvTable'])


class Job(object):
    def __init__(self, data=None):
        data = data or {}
        self.id = data.get('id', '')
        self.script = Script(data.get('script', {}))
        self.script_config = ScriptConfig(data.get('config', {}))
        self.trigger = Trigger(data.get('trigger', {}))
        self.output = Output(data.get('output', {}))


class Config(object):
    def __init__(self, data=None):
        data = data or {}
        self.job_list = []
        for j in data.get('jobList', []):
            self.job_list.append(Job(j))


# Global variables
_cron_dir_prefix = "/usr/whatap/infra/cronjob"
_config_path = "/usr/whatap/infra/cronjob/cronjob.json"
_job_status_map = {}
_job_status_lock = threading.Lock()
_scheduled_jobs = {}  # job_id -> next_run_time
_running = False
_cronjob_thread = None


def _auto_parse(s):
    s = s.strip()

    try:
        return DecimalValue(int(s))
    except ValueError:
        pass

    try:
        return FloatValue(float(s))
    except ValueError:
        pass

    return TextValue(s)


def _resolve_escapes(s):
    if not s:
        return s

    result = []
    i = 0
    while i < len(s):
        if s[i] == '\\' and i + 1 < len(s):
            next_ch = s[i + 1]
            if next_ch == 't':
                result.append(u'\t')
                i += 2
            elif next_ch == 'n':
                result.append(u'\n')
                i += 2
            elif next_ch == 'r':
                result.append(u'\r')
                i += 2
            elif next_ch == '\\':
                result.append(u'\\')
                i += 2
            elif next_ch == 'u' and i + 5 < len(s):
                hex_str = s[i + 2:i + 6]
                try:
                    result.append(unichr(int(hex_str, 16)))
                    i += 6
                except ValueError:
                    result.append(s[i])
                    i += 1
            else:
                result.append(s[i])
                i += 1
        else:
            result.append(s[i])
            i += 1

    return u''.join(result)


def _parse_delimited(csv_str, delimiter):
    result = []
    lines = csv_str.splitlines()
    if not lines:
        return result

    headers = [h.strip() for h in lines[0].split(delimiter)]

    for line in lines[1:]:
        if not line.strip():
            continue
        parts = line.split(delimiter)
        row_data = {}
        for i, val in enumerate(parts):
            if i < len(headers):
                row_data[headers[i]] = _auto_parse(val)
        result.append(row_data)

    return result


def _parse_csv(csv_str, delimiter='|'):
    result = []

    encoded = isinstance(csv_str, unicode)
    if encoded:
        csv_str = csv_str.encode('utf-8')

    reader = csv.reader(StringIO(csv_str), delimiter=str(delimiter))
    rows = list(reader)

    if len(rows) < 1:
        return result

    headers = [
        (h.decode('utf-8') if isinstance(h, str) else h).strip()
        for h in rows[0]
    ]

    for row in rows[1:]:
        if not row:
            continue
        row_data = {}
        for i, val in enumerate(row):
            if i < len(headers):
                key = headers[i]
                v = val.decode('utf-8') if isinstance(val, str) else val
                row_data[key] = _auto_parse(v)
        result.append(row_data)

    return result


def _load_config(path):
    if not os.path.exists(path):
        dir_path = os.path.dirname(path)
        if not os.path.exists(dir_path):
            try:
                os.makedirs(dir_path)
            except OSError as e:
                logging_util.error("[CronJob] Failed to create directory: {}".format(e))
                return Config()

        default_cfg = {"jobList": []}
        try:
            with open(path, 'w') as f:
                json.dump(default_cfg, f, indent=2)
            logging_util.info("[CronJob] Created default config file: {}".format(path))
        except Exception as e:
            logging_util.error("[CronJob] Failed to create default config: {}".format(e))

        return Config()

    try:
        with open(path, 'r') as f:
            data = json.load(f)
        return Config(data)
    except Exception as e:
        logging_util.error("[CronJob] Failed to load config: {}".format(e))
        return Config()


def _update_job_status(job_id, success, start_time, duration, error=None):
    with _job_status_lock:
        if job_id not in _job_status_map:
            _job_status_map[job_id] = JobStatus(job_id)

        status = _job_status_map[job_id]
        status.last_execute_duration = duration

        if success:
            status.last_execute_status = STATUS_SUCCESS
            status.error_message = ""
        else:
            status.last_execute_status = STATUS_ERROR
            if error:
                status.error_message = str(error)


def _set_job_running(job_id, start_time):
    with _job_status_lock:
        if job_id in _job_status_map:
            status = _job_status_map[job_id]
            status.last_execute_status = STATUS_RUNNING
            status.last_execute_start_time = start_time


def _initialize_job_status(job, spec, schedule_type):
    with _job_status_lock:
        if job.id not in _job_status_map:
            _job_status_map[job.id] = JobStatus(job.id)

        status = _job_status_map[job.id]
        status.schedule = spec
        status.schedule_type = schedule_type


def _remove_job_status(job_id):
    with _job_status_lock:
        if job_id in _job_status_map:
            del _job_status_map[job_id]


def _cleanup_orphaned_job_status(active_job_ids):
    with _job_status_lock:
        active_set = set(active_job_ids)
        orphaned = [jid for jid in _job_status_map if jid not in active_set]
        for jid in orphaned:
            del _job_status_map[jid]
            logging_util.debug("[CronJob] Removed orphaned job status: {}".format(jid))


class TimeoutException(Exception):
    pass


def _run_command_with_timeout(script_path, timeout):
    result = ""
    error = None
    output_data = [None]
    proc_error = [None]

    try:
        proc = subprocess.Popen(
            [script_path],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            shell=False
        )

        def communicate_thread():
            try:
                output_data[0], _ = proc.communicate()
            except Exception as e:
                proc_error[0] = e

        comm_thread = threading.Thread(target=communicate_thread)
        comm_thread.start()
        comm_thread.join(timeout)

        if comm_thread.is_alive():
            proc.kill()
            proc.wait()
            error = TimeoutException("Timeout after {} seconds".format(timeout))
        elif proc_error[0]:
            error = proc_error[0]
        else:
            output = output_data[0]
            if output:
                if isinstance(output, bytes):
                    result = output.decode('utf-8', 'replace')
                else:
                    result = output
            else:
                result = ""

    except Exception as e:
        error = e

    return result, error


def _process_result(job, result, error, start_time):
    end_time = dateutil.now()
    duration = end_time - start_time

    _update_job_status(job.id, error is None, start_time, duration, error)

    if error:
        logging_util.error("[CronJob] Job {} failed: {}".format(job.id, error))
        return

    secu = secure.GetSecurityMaster()

    script_data = []
    if job.script.type == 0 or job.script.type == 1:
        raw_deli = job.script.delimiter if job.script.delimiter else u'|'
        delimiter = _resolve_escapes(raw_deli)
        if not isinstance(delimiter, unicode):
            delimiter = delimiter.decode('utf-8')
        raw_deli_u = raw_deli if isinstance(raw_deli, unicode) else raw_deli.decode('utf-8')

        # Resolved \n/\r conflict with row separation (csv EOL, splitlines).
        # Fall back to the literal 2-char form so `echo "...\n..."` (no -e) parses.
        if delimiter in (u'\n', u'\r'):
            delimiter = raw_deli_u
        elif raw_deli_u != delimiter and result:
            # Delimiter was given via escape (e.g., "\t"). If the resolved char
            # isn't in the output, the script likely emitted the literal escape
            # form (`echo` without -e). Normalize it before parsing.
            if isinstance(result, bytes):
                result = result.decode('utf-8', 'replace')
            if delimiter not in result:
                result = result.replace(raw_deli_u, delimiter)

        try:
            if len(delimiter) == 1 and ord(delimiter) < 128:
                script_data = _parse_csv(result, str(delimiter))
            else:
                script_data = _parse_delimited(result, delimiter)
        except Exception as e:
            logging_util.error("[CronJob] Parse error for job {}: {}".format(job.id, e))
            return

    if not script_data:
        logging_util.debug("[CronJob] No data parsed for job {}".format(job.id))
        return

    length = len(script_data)

    tg = job.output.tag_count
    if tg:
        category_name = "custom_" + tg.category_name if tg.category_name else "custom_" + job.script.file_name

        if tg.fold_rec_type == 2:
            p = tagcount_pack.TagCountPack()
            p.category = category_name
            p.pcode = secu.pcode
            p.oid = secu.oid
            p.time = dateutil.now()
            p.putTag("oname", str(secu.oname))
            p.putTag("!rectype", "2")

            list_value_map = {}
            for row_data in script_data:
                for k, v in row_data.items():
                    k = k.strip()
                    if k not in list_value_map:
                        list_value_map[k] = ListValue()
                    list_value_map[k].addValue(v)

            if list_value_map:
                for k, v in list_value_map.items():
                    p.fields.put(k, v)
                logging_util.debug("[CronJob] Sending TagCount (folded) for job {}".format(job.id))
                data.SendHide(p)
        else:
            for i, row_data in enumerate(script_data):
                p = tagcount_pack.TagCountPack()
                p.category = category_name
                p.pcode = secu.pcode
                p.oid = secu.oid
                p.time = dateutil.now()
                p.putTag("oname", str(secu.oname))

                if length > 1:
                    p.putTag("row", str(i))

                for k, v in row_data.items():
                    k = k.strip()
                    p.fields.put(k, v)

                logging_util.debug("[CronJob] Sending TagCount for job {}, row {}".format(job.id, i))
                data.SendHide(p)

    kv = job.output.kv_table
    if kv:
        category_name = "custom_" + kv.category_name if kv.category_name else "custom_" + job.script.file_name

        for i, row_data in enumerate(script_data):
            p = param_pack.ParamPack()
            p.pcode = secu.pcode
            p.oid = secu.oid
            p.time = dateutil.now()
            p.id = net.PARAM_INFRA_HOSTINFO if hasattr(net, 'PARAM_INFRA_HOSTINFO') else 0

            inventory_map = MapValue()
            for k, v in row_data.items():
                k = k.strip()
                inventory_map.put(k, v)

            key_data = "{}-{}-{}-{}".format(secu.oid, p.time, job.id, i if length > 1 else "")
            pk = HashUtil.hash(key_data.encode('utf-8'))

            p.put("table", TextValue(category_name))
            p.put("key", DecimalValue(pk))
            p.put("data", inventory_map)

            logging_util.debug("[CronJob] Sending KvTable for job {}, row {}".format(job.id, i))
            data.SendHide(p)

    logging_util.debug("[CronJob] Processed result for job {}: {} rows".format(job.id, length))


def _run_job(job, timeout):
    start_time = dateutil.now()
    _set_job_running(job.id, start_time)

    script_path = os.path.join(_cron_dir_prefix, job.script.file_name)

    if not os.path.exists(script_path):
        error = Exception("Script not found: {}".format(script_path))
        _process_result(job, "", error, start_time)
        return

    logging_util.debug("[CronJob] Running job {}: {}".format(job.id, script_path))

    result, error = _run_command_with_timeout(script_path, timeout)
    _process_result(job, result, error, start_time)


def _calculate_timeout(job, default_timeout):
    timeout = default_timeout

    if job.trigger.cron:
        if timeout > 60:
            timeout = 60
    elif job.trigger.interval:
        if timeout > job.trigger.interval:
            timeout = job.trigger.interval

    return timeout


def _cronjob_loop():
    global _running, _config_path, _cron_dir_prefix, _scheduled_jobs

    conf = config.GetConfig()
    _cron_dir_prefix = getattr(conf, 'CronjobPath', '/usr/whatap/infra/cronjob')
    _config_path = os.path.join(_cron_dir_prefix, getattr(conf, 'CronjobConf', 'cronjob.json'))
    default_timeout = getattr(conf, 'CronjobTimeout', 5)

    last_mtime = None
    jobs = []

    while _running:
        try:
            try:
                if not os.path.exists(_config_path):
                    cfg = _load_config(_config_path)
                    jobs = cfg.job_list
                    last_mtime = os.path.getmtime(_config_path) if os.path.exists(_config_path) else 0

                    active_job_ids = [j.id for j in jobs]
                    _cleanup_orphaned_job_status(active_job_ids)

                    _scheduled_jobs = {}
                    for job in jobs:
                        if job.trigger.cron:
                            _initialize_job_status(job, job.trigger.cron, SCHEDULE_TYPE_CRON)
                            try:
                                cron = get_croniter(job.trigger.cron, datetime.now())
                                _scheduled_jobs[job.id] = {
                                    'type': SCHEDULE_TYPE_CRON,
                                    'cron': job.trigger.cron,
                                    'next_run': cron.get_next(datetime),
                                    'job': job
                                }
                            except Exception as e:
                                logging_util.error("[CronJob] Failed to parse cron spec '{}': {}".format(job.trigger.cron, e))
                        elif job.trigger.interval:
                            _initialize_job_status(job, str(job.trigger.interval), SCHEDULE_TYPE_INTERVAL)
                            _scheduled_jobs[job.id] = {
                                'type': SCHEDULE_TYPE_INTERVAL,
                                'interval': job.trigger.interval,
                                'next_run': datetime.now(),
                                'job': job
                            }

                    logging_util.debug("[CronJob] Config created and loaded ({} jobs)".format(len(jobs)))

                elif os.path.exists(_config_path):
                    mtime = os.path.getmtime(_config_path)
                    if last_mtime is None or mtime > last_mtime:
                        cfg = _load_config(_config_path)
                        jobs = cfg.job_list
                        last_mtime = mtime

                        active_job_ids = [j.id for j in jobs]
                        _cleanup_orphaned_job_status(active_job_ids)

                        _scheduled_jobs = {}
                        for job in jobs:
                            if job.trigger.cron:
                                _initialize_job_status(job, job.trigger.cron, SCHEDULE_TYPE_CRON)
                                try:
                                    cron = get_croniter(job.trigger.cron, datetime.now())
                                    _scheduled_jobs[job.id] = {
                                        'type': SCHEDULE_TYPE_CRON,
                                        'cron': job.trigger.cron,
                                        'next_run': cron.get_next(datetime),
                                        'job': job
                                    }
                                except Exception as e:
                                    logging_util.error("[CronJob] Failed to parse cron spec '{}': {}".format(job.trigger.cron, e))
                            elif job.trigger.interval:
                                _initialize_job_status(job, str(job.trigger.interval), SCHEDULE_TYPE_INTERVAL)
                                _scheduled_jobs[job.id] = {
                                    'type': SCHEDULE_TYPE_INTERVAL,
                                    'interval': job.trigger.interval,
                                    'next_run': datetime.now(),
                                    'job': job
                                }

                        logging_util.debug("[CronJob] Config reloaded ({} jobs)".format(len(jobs)))
            except Exception as e:
                logging_util.error("[CronJob] Config load error: {}".format(e))

            now = datetime.now()
            for job_id, schedule_info in _scheduled_jobs.items():
                try:
                    if now >= schedule_info['next_run']:
                        job = schedule_info['job']
                        timeout = _calculate_timeout(job, default_timeout)

                        t = threading.Thread(target=_run_job, args=(job, timeout))
                        t.daemon = True
                        t.start()

                        if schedule_info['type'] == SCHEDULE_TYPE_CRON:
                            cron = get_croniter(schedule_info['cron'], now)
                            schedule_info['next_run'] = cron.get_next(datetime)
                        elif schedule_info['type'] == SCHEDULE_TYPE_INTERVAL:
                            from datetime import timedelta
                            schedule_info['next_run'] = now + timedelta(seconds=schedule_info['interval'])
                except Exception as e:
                    logging_util.error("[CronJob] Job execution error for {}: {}".format(job_id, e))

            time.sleep(CONFIG_POLL_INTERVAL)

        except Exception as e:
            logging_util.error("[CronJob] Main loop error: {}".format(e))
            time.sleep(CONFIG_POLL_INTERVAL)


def start_cronjob():
    global _running, _cronjob_thread

    conf = config.GetConfig()

    if not getattr(conf, 'CronjobEnabled', False):
        logging_util.info("[CronJob] Disabled by configuration")
        return

    if croniter is None:
        logging_util.info("[CronJob] croniter library not available, using built-in SimpleCroniter")

    _running = True
    _cronjob_thread = threading.Thread(target=_cronjob_loop)
    _cronjob_thread.daemon = True
    _cronjob_thread.start()

    logging_util.info("[CronJob] Started")


def stop_cronjob():
    global _running
    _running = False
    logging_util.info("[CronJob] Stopped")


def get_cronjob_status():
    with _job_status_lock:
        return [
            {
                'id': s.id,
                'schedule': s.schedule,
                'scheduleType': s.schedule_type,
                'lastExecuteStartTime': s.last_execute_start_time,
                'lastExecuteStatus': s.last_execute_status,
                'lastExecuteDuration': s.last_execute_duration,
                'errorMessage': s.error_message
            }
            for s in _job_status_map.values()
        ]


def get_cron_config_path(path=""):
    conf = config.GetConfig()
    cron_dir = getattr(conf, 'CronjobPath', '/usr/whatap/infra/cronjob')
    if path:
        return os.path.join(cron_dir, path)
    return cron_dir


def _safe_write_atomic(path, content, make_executable=False):
    dir_path = os.path.dirname(path)
    base_name = os.path.basename(path)
    tmp_path = os.path.join(dir_path, "." + base_name + ".tmp")

    if not os.path.exists(dir_path):
        os.makedirs(dir_path, 0o755)

    with open(tmp_path, 'w') as f:
        f.write(content)
        f.flush()
        os.fsync(f.fileno())

    if make_executable:
        os.chmod(tmp_path, 0o755)
    else:
        os.chmod(tmp_path, 0o644)

    if os.path.exists(path):
        os.remove(path)
    os.rename(tmp_path, path)


def _safe_write_json_atomic(path, data):
    content = json.dumps(data, indent=2)
    if not content.strip().startswith('{'):
        raise ValueError("JSON must be an object at top-level")
    _safe_write_atomic(path, content, make_executable=False)


def save_script_file(new_cfg):
    conf = config.GetConfig()
    cron_dir = getattr(conf, 'CronjobPath', '/usr/whatap/infra/cronjob')

    job_list = new_cfg.get('jobList', [])
    for job in job_list:
        job_id = job.get('id', '')
        script = job.get('script', {})
        script_config = job.get('config', {})

        file_name = script.get('fileName', '')
        content_b64 = script.get('content', '')

        if file_name and content_b64:
            try:
                script_content = base64.b64decode(content_b64)
                if isinstance(script_content, bytes):
                    script_content = script_content.decode('utf-8', 'replace')
            except Exception as e:
                logging_util.error("[CronJob] SaveScriptFile invalid base64 content ({} : {}): {}".format(
                    job_id, file_name, e))
                raise e

            script_path = os.path.join(cron_dir, file_name)
            logging_util.info("[CronJob] SaveScriptFile scriptPath: {}, content size: {}".format(
                script_path, len(script_content)))

            try:
                _safe_write_atomic(script_path, script_content, make_executable=True)
                logging_util.info("[CronJob] SaveScriptFile success: {}".format(script_path))
            except Exception as e:
                logging_util.error("[CronJob] SaveScriptFile failed ({} : {}): {}".format(
                    job_id, file_name, e))
                raise e

            script['content'] = ""

        config_file_name = script_config.get('fileName', '')
        config_content_b64 = script_config.get('content', '')

        if config_file_name and config_content_b64:
            try:
                config_content = base64.b64decode(config_content_b64)
                if isinstance(config_content, bytes):
                    config_content = config_content.decode('utf-8', 'replace')
            except Exception as e:
                logging_util.error("[CronJob] SaveScriptConfig invalid base64 content ({} : {}): {}".format(
                    job_id, config_file_name, e))
                raise e

            config_path = os.path.join(cron_dir, config_file_name)
            try:
                _safe_write_atomic(config_path, config_content, make_executable=False)
                logging_util.info("[CronJob] SaveScriptConfig success: {}".format(config_path))
            except Exception as e:
                logging_util.error("[CronJob] SaveScriptConfig failed ({} : {}): {}".format(
                    job_id, config_file_name, e))
                raise e

            script_config['content'] = ""


def save_cron_config(new_cfg):
    global _config_path

    conf = config.GetConfig()
    cron_dir = getattr(conf, 'CronjobPath', '/usr/whatap/infra/cronjob')
    config_path = os.path.join(cron_dir, getattr(conf, 'CronjobConf', 'cronjob.json'))

    logging_util.info("[CronJob] SaveCronConfig called")

    old_cfg = _load_config(config_path)
    old_job_list = old_cfg.job_list if hasattr(old_cfg, 'job_list') else []

    idx = {}
    for i, j in enumerate(old_job_list):
        idx[j.id] = i

    new_job_list = new_cfg.get('jobList', [])
    merged_jobs = list(old_job_list)

    for new_job_data in new_job_list:
        new_job = Job(new_job_data)
        new_job.script.content = ""
        if new_job.script_config:
            new_job.script_config.content = ""

        if new_job.id in idx:
            merged_jobs[idx[new_job.id]] = new_job
        else:
            merged_jobs.append(new_job)

    merged_cfg = {
        'jobList': [
            {
                'id': j.id,
                'script': {
                    'fileName': j.script.file_name,
                    'type': j.script.type,
                    'delimiter': j.script.delimiter,
                    'content': ''
                },
                'config': {
                    'fileName': j.script_config.file_name if j.script_config else '',
                    'content': ''
                },
                'trigger': {
                    'cron': j.trigger.cron,
                    'interval': j.trigger.interval
                } if j.trigger else {},
                'output': {
                    'tagCount': {
                        'categoryName': j.output.tag_count.category_name,
                        'columnMap': {k: {'type': v.type, 'required': v.required}
                                      for k, v in j.output.tag_count.column_map.items()},
                        'foldRecType': j.output.tag_count.fold_rec_type
                    } if j.output and j.output.tag_count else None,
                    'kvTable': {
                        'categoryName': j.output.kv_table.category_name,
                        'columnMap': {k: {'type': v.type, 'required': v.required}
                                      for k, v in j.output.kv_table.column_map.items()}
                    } if j.output and j.output.kv_table else None
                } if j.output else {}
            }
            for j in merged_jobs
        ]
    }

    for job_data in merged_cfg['jobList']:
        if job_data['output'].get('tagCount') is None:
            del job_data['output']['tagCount']
        if job_data['output'].get('kvTable') is None:
            del job_data['output']['kvTable']
        if job_data['trigger'].get('cron') is None:
            del job_data['trigger']['cron']
        if job_data['trigger'].get('interval') is None:
            del job_data['trigger']['interval']

    logging_util.info("[CronJob] SaveCronConfig merged config to save")

    try:
        _safe_write_json_atomic(config_path, merged_cfg)
        logging_util.info("[CronJob] SaveCronConfig success: {}".format(config_path))
    except Exception as e:
        logging_util.error("[CronJob] SaveCronConfig failed: {}".format(e))
        raise e


def delete_cron_config(job_ids):
    global _config_path

    conf = config.GetConfig()
    cron_dir = getattr(conf, 'CronjobPath', '/usr/whatap/infra/cronjob')
    config_path = os.path.join(cron_dir, getattr(conf, 'CronjobConf', 'cronjob.json'))

    old_cfg = _load_config(config_path)
    old_job_list = old_cfg.job_list if hasattr(old_cfg, 'job_list') else []

    jobs_to_delete = set(job_ids) if isinstance(job_ids, list) else set(job_ids.keys())

    remaining_jobs = []
    for job in old_job_list:
        if job.id in jobs_to_delete:
            _remove_job_status(job.id)
            logging_util.debug("[CronJob] DeleteCronConfig removing job: {}".format(job.id))
        else:
            remaining_jobs.append(job)

    remaining_cfg = {
        'jobList': [
            {
                'id': j.id,
                'script': {
                    'fileName': j.script.file_name,
                    'type': j.script.type,
                    'delimiter': j.script.delimiter,
                    'content': ''
                },
                'config': {
                    'fileName': j.script_config.file_name if j.script_config else '',
                    'content': ''
                },
                'trigger': {
                    'cron': j.trigger.cron,
                    'interval': j.trigger.interval
                } if j.trigger else {},
                'output': {
                    'tagCount': {
                        'categoryName': j.output.tag_count.category_name,
                        'columnMap': {k: {'type': v.type, 'required': v.required}
                                      for k, v in j.output.tag_count.column_map.items()},
                        'foldRecType': j.output.tag_count.fold_rec_type
                    } if j.output and j.output.tag_count else None,
                    'kvTable': {
                        'categoryName': j.output.kv_table.category_name,
                        'columnMap': {k: {'type': v.type, 'required': v.required}
                                      for k, v in j.output.kv_table.column_map.items()}
                    } if j.output and j.output.kv_table else None
                } if j.output else {}
            }
            for j in remaining_jobs
        ]
    }

    for job_data in remaining_cfg['jobList']:
        if job_data['output'].get('tagCount') is None:
            del job_data['output']['tagCount']
        if job_data['output'].get('kvTable') is None:
            del job_data['output']['kvTable']
        if job_data['trigger'].get('cron') is None:
            del job_data['trigger']['cron']
        if job_data['trigger'].get('interval') is None:
            del job_data['trigger']['interval']

    try:
        _safe_write_json_atomic(config_path, remaining_cfg)
        logging_util.info("[CronJob] DeleteCronConfig success: {}".format(config_path))
    except Exception as e:
        logging_util.error("[CronJob] DeleteCronConfig failed: {}".format(e))
        raise e



#! /usr/bin/env python

"""Non-terminal symbols of Python grammar (from "graminit.h")."""

#  This file is automatically generated; please don't muck it up!
#
#  To update the symbols in this file, 'cd' to the top directory of
#  the python source tree after building the interpreter and run:
#
#    ./python Lib/symbol.py

#--start constants--
single_input = 256
file_input = 257
eval_input = 258
decorator = 259
decorators = 260
decorated = 261
funcdef = 262
parameters = 263
varargslist = 264
fpdef = 265
fplist = 266
stmt = 267
simple_stmt = 268
small_stmt = 269
expr_stmt = 270
augassign = 271
print_stmt = 272
del_stmt = 273
pass_stmt = 274
flow_stmt = 275
break_stmt = 276
continue_stmt = 277
return_stmt = 278
yield_stmt = 279
raise_stmt = 280
import_stmt = 281
import_name = 282
import_from = 283
import_as_name = 284
dotted_as_name = 285
import_as_names = 286
dotted_as_names = 287
dotted_name = 288
global_stmt = 289
exec_stmt = 290
assert_stmt = 291
compound_stmt = 292
if_stmt = 293
while_stmt = 294
for_stmt = 295
try_stmt = 296
with_stmt = 297
with_item = 298
except_clause = 299
suite = 300
testlist_safe = 301
old_test = 302
old_lambdef = 303
test = 304
or_test = 305
and_test = 306
not_test = 307
comparison = 308
comp_op = 309
expr = 310
xor_expr = 311
and_expr = 312
shift_expr = 313
arith_expr = 314
term = 315
factor = 316
power = 317
atom = 318
listmaker = 319
testlist_comp = 320
lambdef = 321
trailer = 322
subscriptlist = 323
subscript = 324
sliceop = 325
exprlist = 326
testlist = 327
dictorsetmaker = 328
classdef = 329
arglist = 330
argument = 331
list_iter = 332
list_for = 333
list_if = 334
comp_iter = 335
comp_for = 336
comp_if = 337
testlist1 = 338
encoding_decl = 339
yield_expr = 340
#--end constants--

sym_name = {}
for _name, _value in globals().items():
    if type(_value) is type(0):
        sym_name[_value] = _name


def main():
    import sys
    import token
    if len(sys.argv) == 1:
        sys.argv = sys.argv + ["Include/graminit.h", "Lib/symbol.py"]
    token.main()

if __name__ == "__main__":
    main()



######################################################################
#  This file should be kept compatible with Python 2.3, see PEP 291. #
######################################################################
"""create and manipulate C data types in Python"""

import os as _os, sys as _sys

__version__ = "1.1.0"

from _ctypes import Union, Structure, Array
from _ctypes import _Pointer
from _ctypes import CFuncPtr as _CFuncPtr
from _ctypes import __version__ as _ctypes_version
from _ctypes import RTLD_LOCAL, RTLD_GLOBAL
from _ctypes import ArgumentError

from struct import calcsize as _calcsize

if __version__ != _ctypes_version:
    raise Exception("Version number mismatch", __version__, _ctypes_version)

if _os.name in ("nt", "ce"):
    from _ctypes import FormatError

DEFAULT_MODE = RTLD_LOCAL
if _os.name == "posix" and _sys.platform == "darwin":
    # On OS X 10.3, we use RTLD_GLOBAL as default mode
    # because RTLD_LOCAL does not work at least on some
    # libraries.  OS X 10.3 is Darwin 7, so we check for
    # that.

    if int(_os.uname()[2].split('.')[0]) < 8:
        DEFAULT_MODE = RTLD_GLOBAL

from _ctypes import FUNCFLAG_CDECL as _FUNCFLAG_CDECL, \
     FUNCFLAG_PYTHONAPI as _FUNCFLAG_PYTHONAPI, \
     FUNCFLAG_USE_ERRNO as _FUNCFLAG_USE_ERRNO, \
     FUNCFLAG_USE_LASTERROR as _FUNCFLAG_USE_LASTERROR

"""
WINOLEAPI -> HRESULT
WINOLEAPI_(type)

STDMETHODCALLTYPE

STDMETHOD(name)
STDMETHOD_(type, name)

STDAPICALLTYPE
"""

def create_string_buffer(init, size=None):
    """create_string_buffer(aString) -> character array
    create_string_buffer(anInteger) -> character array
    create_string_buffer(aString, anInteger) -> character array
    """
    if isinstance(init, (str, unicode)):
        if size is None:
            size = len(init)+1
        buftype = c_char * size
        buf = buftype()
        buf.value = init
        return buf
    elif isinstance(init, (int, long)):
        buftype = c_char * init
        buf = buftype()
        return buf
    raise TypeError(init)

def c_buffer(init, size=None):
##    "deprecated, use create_string_buffer instead"
##    import warnings
##    warnings.warn("c_buffer is deprecated, use create_string_buffer instead",
##                  DeprecationWarning, stacklevel=2)
    return create_string_buffer(init, size)

_c_functype_cache = {}
def CFUNCTYPE(restype, *argtypes, **kw):
    """CFUNCTYPE(restype, *argtypes,
                 use_errno=False, use_last_error=False) -> function prototype.

    restype: the result type
    argtypes: a sequence specifying the argument types

    The function prototype can be called in different ways to create a
    callable object:

    prototype(integer address) -> foreign function
    prototype(callable) -> create and return a C callable function from callable
    prototype(integer index, method name[, paramflags]) -> foreign function calling a COM method
    prototype((ordinal number, dll object)[, paramflags]) -> foreign function exported by ordinal
    prototype((function name, dll object)[, paramflags]) -> foreign function exported by name
    """
    flags = _FUNCFLAG_CDECL
    if kw.pop("use_errno", False):
        flags |= _FUNCFLAG_USE_ERRNO
    if kw.pop("use_last_error", False):
        flags |= _FUNCFLAG_USE_LASTERROR
    if kw:
        raise ValueError("unexpected keyword argument(s) %s" % kw.keys())
    try:
        return _c_functype_cache[(restype, argtypes, flags)]
    except KeyError:
        class CFunctionType(_CFuncPtr):
            _argtypes_ = argtypes
            _restype_ = restype
            _flags_ = flags
        _c_functype_cache[(restype, argtypes, flags)] = CFunctionType
        return CFunctionType

if _os.name in ("nt", "ce"):
    from _ctypes import LoadLibrary as _dlopen
    from _ctypes import FUNCFLAG_STDCALL as _FUNCFLAG_STDCALL
    if _os.name == "ce":
        # 'ce' doesn't have the stdcall calling convention
        _FUNCFLAG_STDCALL = _FUNCFLAG_CDECL

    _win_functype_cache = {}
    def WINFUNCTYPE(restype, *argtypes, **kw):
        # docstring set later (very similar to CFUNCTYPE.__doc__)
        flags = _FUNCFLAG_STDCALL
        if kw.pop("use_errno", False):
            flags |= _FUNCFLAG_USE_ERRNO
        if kw.pop("use_last_error", False):
            flags |= _FUNCFLAG_USE_LASTERROR
        if kw:
            raise ValueError("unexpected keyword argument(s) %s" % kw.keys())
        try:
            return _win_functype_cache[(restype, argtypes, flags)]
        except KeyError:
            class WinFunctionType(_CFuncPtr):
                _argtypes_ = argtypes
                _restype_ = restype
                _flags_ = flags
            _win_functype_cache[(restype, argtypes, flags)] = WinFunctionType
            return WinFunctionType
    if WINFUNCTYPE.__doc__:
        WINFUNCTYPE.__doc__ = CFUNCTYPE.__doc__.replace("CFUNCTYPE", "WINFUNCTYPE")

elif _os.name == "posix":
    from _ctypes import dlopen as _dlopen

from _ctypes import sizeof, byref, addressof, alignment, resize
from _ctypes import get_errno, set_errno
from _ctypes import _SimpleCData

def _check_size(typ, typecode=None):
    # Check if sizeof(ctypes_type) against struct.calcsize.  This
    # should protect somewhat against a misconfigured libffi.
    from struct import calcsize
    if typecode is None:
        # Most _type_ codes are the same as used in struct
        typecode = typ._type_
    actual, required = sizeof(typ), calcsize(typecode)
    if actual != required:
        raise SystemError("sizeof(%s) wrong: %d instead of %d" % \
                          (typ, actual, required))

class py_object(_SimpleCData):
    _type_ = "O"
    def __repr__(self):
        try:
            return super(py_object, self).__repr__()
        except ValueError:
            return "%s(<NULL>)" % type(self).__name__
_check_size(py_object, "P")

class c_short(_SimpleCData):
    _type_ = "h"
_check_size(c_short)

class c_ushort(_SimpleCData):
    _type_ = "H"
_check_size(c_ushort)

class c_long(_SimpleCData):
    _type_ = "l"
_check_size(c_long)

class c_ulong(_SimpleCData):
    _type_ = "L"
_check_size(c_ulong)

if _calcsize("i") == _calcsize("l"):
    # if int and long have the same size, make c_int an alias for c_long
    c_int = c_long
    c_uint = c_ulong
else:
    class c_int(_SimpleCData):
        _type_ = "i"
    _check_size(c_int)

    class c_uint(_SimpleCData):
        _type_ = "I"
    _check_size(c_uint)

class c_float(_SimpleCData):
    _type_ = "f"
_check_size(c_float)

class c_double(_SimpleCData):
    _type_ = "d"
_check_size(c_double)

class c_longdouble(_SimpleCData):
    _type_ = "g"
if sizeof(c_longdouble) == sizeof(c_double):
    c_longdouble = c_double

if _calcsize("l") == _calcsize("q"):
    # if long and long long have the same size, make c_longlong an alias for c_long
    c_longlong = c_long
    c_ulonglong = c_ulong
else:
    class c_longlong(_SimpleCData):
        _type_ = "q"
    _check_size(c_longlong)

    class c_ulonglong(_SimpleCData):
        _type_ = "Q"
    ##    def from_param(cls, val):
    ##        return ('d', float(val), val)
    ##    from_param = classmethod(from_param)
    _check_size(c_ulonglong)

class c_ubyte(_SimpleCData):
    _type_ = "B"
c_ubyte.__ctype_le__ = c_ubyte.__ctype_be__ = c_ubyte
# backward compatibility:
##c_uchar = c_ubyte
_check_size(c_ubyte)

class c_byte(_SimpleCData):
    _type_ = "b"
c_byte.__ctype_le__ = c_byte.__ctype_be__ = c_byte
_check_size(c_byte)

class c_char(_SimpleCData):
    _type_ = "c"
c_char.__ctype_le__ = c_char.__ctype_be__ = c_char
_check_size(c_char)

class c_char_p(_SimpleCData):
    _type_ = "z"
    if _os.name == "nt":
        def __repr__(self):
            if not windll.kernel32.IsBadStringPtrA(self, -1):
                return "%s(%r)" % (self.__class__.__name__, self.value)
            return "%s(%s)" % (self.__class__.__name__, cast(self, c_void_p).value)
    else:
        def __repr__(self):
            return "%s(%s)" % (self.__class__.__name__, cast(self, c_void_p).value)
_check_size(c_char_p, "P")

class c_void_p(_SimpleCData):
    _type_ = "P"
c_voidp = c_void_p # backwards compatibility (to a bug)
_check_size(c_void_p)

class c_bool(_SimpleCData):
    _type_ = "?"

from _ctypes import POINTER, pointer, _pointer_type_cache

def _reset_cache():
    _pointer_type_cache.clear()
    _c_functype_cache.clear()
    if _os.name in ("nt", "ce"):
        _win_functype_cache.clear()
    # _SimpleCData.c_wchar_p_from_param
    POINTER(c_wchar).from_param = c_wchar_p.from_param
    # _SimpleCData.c_char_p_from_param
    POINTER(c_char).from_param = c_char_p.from_param
    _pointer_type_cache[None] = c_void_p
    # XXX for whatever reasons, creating the first instance of a callback
    # function is needed for the unittests on Win64 to succeed.  This MAY
    # be a compiler bug, since the problem occurs only when _ctypes is
    # compiled with the MS SDK compiler.  Or an uninitialized variable?
    CFUNCTYPE(c_int)(lambda: None)

try:
    from _ctypes import set_conversion_mode
except ImportError:
    pass
else:
    if _os.name in ("nt", "ce"):
        set_conversion_mode("mbcs", "ignore")
    else:
        set_conversion_mode("ascii", "strict")

    class c_wchar_p(_SimpleCData):
        _type_ = "Z"

    class c_wchar(_SimpleCData):
        _type_ = "u"

    def create_unicode_buffer(init, size=None):
        """create_unicode_buffer(aString) -> character array
        create_unicode_buffer(anInteger) -> character array
        create_unicode_buffer(aString, anInteger) -> character array
        """
        if isinstance(init, (str, unicode)):
            if size is None:
                size = len(init)+1
            buftype = c_wchar * size
            buf = buftype()
            buf.value = init
            return buf
        elif isinstance(init, (int, long)):
            buftype = c_wchar * init
            buf = buftype()
            return buf
        raise TypeError(init)

# XXX Deprecated
def SetPointerType(pointer, cls):
    if _pointer_type_cache.get(cls, None) is not None:
        raise RuntimeError("This type already exists in the cache")
    if id(pointer) not in _pointer_type_cache:
        raise RuntimeError("What's this???")
    pointer.set_type(cls)
    _pointer_type_cache[cls] = pointer
    del _pointer_type_cache[id(pointer)]

# XXX Deprecated
def ARRAY(typ, len):
    return typ * len

################################################################


class CDLL(object):
    """An instance of this class represents a loaded dll/shared
    library, exporting functions using the standard C calling
    convention (named 'cdecl' on Windows).

    The exported functions can be accessed as attributes, or by
    indexing with the function name.  Examples:

    <obj>.qsort -> callable object
    <obj>['qsort'] -> callable object

    Calling the functions releases the Python GIL during the call and
    reacquires it afterwards.
    """
    _func_flags_ = _FUNCFLAG_CDECL
    _func_restype_ = c_int

    def __init__(self, name, mode=DEFAULT_MODE, handle=None,
                 use_errno=False,
                 use_last_error=False):
        self._name = name
        flags = self._func_flags_
        if use_errno:
            flags |= _FUNCFLAG_USE_ERRNO
        if use_last_error:
            flags |= _FUNCFLAG_USE_LASTERROR

        class _FuncPtr(_CFuncPtr):
            _flags_ = flags
            _restype_ = self._func_restype_
        self._FuncPtr = _FuncPtr

        if handle is None:
            self._handle = _dlopen(self._name, mode)
        else:
            self._handle = handle

    def __repr__(self):
        return "<%s '%s', handle %x at %x>" % \
               (self.__class__.__name__, self._name,
                (self._handle & (_sys.maxint*2 + 1)),
                id(self) & (_sys.maxint*2 + 1))

    def __getattr__(self, name):
        if name.startswith('__') and name.endswith('__'):
            raise AttributeError(name)
        func = self.__getitem__(name)
        setattr(self, name, func)
        return func

    def __getitem__(self, name_or_ordinal):
        func = self._FuncPtr((name_or_ordinal, self))
        if not isinstance(name_or_ordinal, (int, long)):
            func.__name__ = name_or_ordinal
        return func

class PyDLL(CDLL):
    """This class represents the Python library itself.  It allows to
    access Python API functions.  The GIL is not released, and
    Python exceptions are handled correctly.
    """
    _func_flags_ = _FUNCFLAG_CDECL | _FUNCFLAG_PYTHONAPI

if _os.name in ("nt", "ce"):

    class WinDLL(CDLL):
        """This class represents a dll exporting functions using the
        Windows stdcall calling convention.
        """
        _func_flags_ = _FUNCFLAG_STDCALL

    # XXX Hm, what about HRESULT as normal parameter?
    # Mustn't it derive from c_long then?
    from _ctypes import _check_HRESULT, _SimpleCData
    class HRESULT(_SimpleCData):
        _type_ = "l"
        # _check_retval_ is called with the function's result when it
        # is used as restype.  It checks for the FAILED bit, and
        # raises a WindowsError if it is set.
        #
        # The _check_retval_ method is implemented in C, so that the
        # method definition itself is not included in the traceback
        # when it raises an error - that is what we want (and Python
        # doesn't have a way to raise an exception in the caller's
        # frame).
        _check_retval_ = _check_HRESULT

    class OleDLL(CDLL):
        """This class represents a dll exporting functions using the
        Windows stdcall calling convention, and returning HRESULT.
        HRESULT error values are automatically raised as WindowsError
        exceptions.
        """
        _func_flags_ = _FUNCFLAG_STDCALL
        _func_restype_ = HRESULT

class LibraryLoader(object):
    def __init__(self, dlltype):
        self._dlltype = dlltype

    def __getattr__(self, name):
        if name[0] == '_':
            raise AttributeError(name)
        dll = self._dlltype(name)
        setattr(self, name, dll)
        return dll

    def __getitem__(self, name):
        return getattr(self, name)

    def LoadLibrary(self, name):
        return self._dlltype(name)

cdll = LibraryLoader(CDLL)
pydll = LibraryLoader(PyDLL)

if _os.name in ("nt", "ce"):
    pythonapi = PyDLL("python dll", None, _sys.dllhandle)
elif _sys.platform == "cygwin":
    pythonapi = PyDLL("libpython%d.%d.dll" % _sys.version_info[:2])
else:
    pythonapi = PyDLL(None)


if _os.name in ("nt", "ce"):
    windll = LibraryLoader(WinDLL)
    oledll = LibraryLoader(OleDLL)

    if _os.name == "nt":
        GetLastError = windll.kernel32.GetLastError
    else:
        GetLastError = windll.coredll.GetLastError
    from _ctypes import get_last_error, set_last_error

    def WinError(code=None, descr=None):
        if code is None:
            code = GetLastError()
        if descr is None:
            descr = FormatError(code).strip()
        return WindowsError(code, descr)

if sizeof(c_uint) == sizeof(c_void_p):
    c_size_t = c_uint
    c_ssize_t = c_int
elif sizeof(c_ulong) == sizeof(c_void_p):
    c_size_t = c_ulong
    c_ssize_t = c_long
elif sizeof(c_ulonglong) == sizeof(c_void_p):
    c_size_t = c_ulonglong
    c_ssize_t = c_longlong

# functions

from _ctypes import _memmove_addr, _memset_addr, _string_at_addr, _cast_addr

## void *memmove(void *, const void *, size_t);
memmove = CFUNCTYPE(c_void_p, c_void_p, c_void_p, c_size_t)(_memmove_addr)

## void *memset(void *, int, size_t)
memset = CFUNCTYPE(c_void_p, c_void_p, c_int, c_size_t)(_memset_addr)

def PYFUNCTYPE(restype, *argtypes):
    class CFunctionType(_CFuncPtr):
        _argtypes_ = argtypes
        _restype_ = restype
        _flags_ = _FUNCFLAG_CDECL | _FUNCFLAG_PYTHONAPI
    return CFunctionType

_cast = PYFUNCTYPE(py_object, c_void_p, py_object, py_object)(_cast_addr)
def cast(obj, typ):
    return _cast(obj, obj, typ)

_string_at = PYFUNCTYPE(py_object, c_void_p, c_int)(_string_at_addr)
def string_at(ptr, size=-1):
    """string_at(addr[, size]) -> string

    Return the string at addr."""
    return _string_at(ptr, size)

try:
    from _ctypes import _wstring_at_addr
except ImportError:
    pass
else:
    _wstring_at = PYFUNCTYPE(py_object, c_void_p, c_int)(_wstring_at_addr)
    def wstring_at(ptr, size=-1):
        """wstring_at(addr[, size]) -> string

        Return the string at addr."""
        return _wstring_at(ptr, size)


if _os.name in ("nt", "ce"): # COM stuff
    def DllGetClassObject(rclsid, riid, ppv):
        try:
            ccom = __import__("comtypes.server.inprocserver", globals(), locals(), ['*'])
        except ImportError:
            return -2147221231 # CLASS_E_CLASSNOTAVAILABLE
        else:
            return ccom.DllGetClassObject(rclsid, riid, ppv)

    def DllCanUnloadNow():
        try:
            ccom = __import__("comtypes.server.inprocserver", globals(), locals(), ['*'])
        except ImportError:
            return 0 # S_OK
        return ccom.DllCanUnloadNow()

from ctypes._endian import BigEndianStructure, LittleEndianStructure

# Fill in specifically-sized types
c_int8 = c_byte
c_uint8 = c_ubyte
for kind in [c_short, c_int, c_long, c_longlong]:
    if sizeof(kind) == 2: c_int16 = kind
    elif sizeof(kind) == 4: c_int32 = kind
    elif sizeof(kind) == 8: c_int64 = kind
for kind in [c_ushort, c_uint, c_ulong, c_ulonglong]:
    if sizeof(kind) == 2: c_uint16 = kind
    elif sizeof(kind) == 4: c_uint32 = kind
    elif sizeof(kind) == 8: c_uint64 = kind
del(kind)

_reset_cache()



from whatap.value.value_enum import ValueEnum

class Value(object):
    def __init__(self):
        return

    def __repr__(self):
        to_str = '%s: '%(type(self).__name__)
        for key, value in self.__dict__.items():
            if isinstance(value, unicode):
                to_str += '%s=%s, ' % (key, value.encode('utf8'))
            else:
                to_str += '%s=%s, '%(key, str(value))
        return to_str

    def equals(self, object):
        if isinstance(object, Value):
            return self.value == object.value
        return False

    def getValueType(self):
        return 0

    def toObject(self):
        return self

    def write(self, dout):
        return

    def read(self, din):
        return self

def writeValue(out, val ):
	out.writeByte(val.getValueType())
	val.write(out)
	return out

def readValue(in_ ) :
	t = in_.readByte()
	v = ValueEnum.create(t)
	v.read(in_)
	return v



from whatap.pack.pack_enum import PackEnum
from whatap.pack.pack import Pack
from whatap.io.data_outputx import DataOutputX

from whatap.io.data_inputx import DataInputX

class SMDiskPack(Pack):
    def __init__(self):
        Pack.__init__(self)
        self.os = None
        self.disk = []

    def getPackType(self):
        return PackEnum.PACK_SM_DISK_QUATA

    def write(self, dout):
        super(SMDiskPack, self).write(dout)
        dout.writeShort(self.os)
        dout.writeDecimal(len(self.disk))
        for d in self.disk:
            d.write(dout)

    def read(self, din):
        din = DataInputX(din.readBlob())
        super(SMDiskPack, self).read(din)
        self.os = din.readShort()
        diskCount = din.readDecimal()
        for i in range(diskCount):
            d = DiskPerf()
            d.read(din)
            self.disk.append(d)

class DiskPerf:
    def __init__(self):
        self.mountPoint = None
        self.deviceId = None
        self.fileSystem = None
        self.totalSpace = None
        self.usedSpace = None
        self.usedPercent = None
        self.freeSpace = None
        self.freePercent = None
        self.blksize = None
        self.readIops = 0
        self.writeIops = 0
        self.readBps = 0
        self.writeBps = 0
        self.ioPercent = 0
        self.queueLength = 0

    def write(self, doutx ):
        dout= DataOutputX()
        dout.writeInt(self.deviceId)
        dout.writeInt(self.mountPoint)
        dout.writeInt(self.fileSystem)
        dout.writeDecimal(self.freeSpace)
        dout.writeDecimal(self.usedSpace)
        dout.writeDecimal(self.totalSpace)
        dout.writeFloat(self.freePercent)
        dout.writeFloat(self.usedPercent)
        dout.writeInt(self.blksize)
        dout.writeDouble(self.readIops)
        dout.writeDouble(self.writeIops)
        dout.writeDouble(self.readBps)
        dout.writeDouble(self.writeBps)
        dout.writeFloat(self.ioPercent)
        dout.writeInt(1)
        dout.writeFloat(self.queueLength)

        doutx.writeBlob(dout.toByteArray())

    def read(self, dinx):
        din= DataInputX(dinx.readBlob())
        self.deviceId= din.ReadInt()
        self.mountPoint= din.ReadInt()
        self.fileSystem = din.ReadInt()
        self.freeSpace = din.ReadDecimal()
        self.usedSpace = din.ReadDecimal()
        self.totalSpace = din.ReadDecimal()
        self.freePercent = din.ReadFloat()
        self.usedPercent = din.ReadFloat()
        self.blksize = din.ReadInt()
        self.readIops = din.ReadDouble()
        self.writeIops = din.ReadDouble()
        self.readBps = din.ReadDouble()
        self.writeBps = din.ReadDouble()
        self.ioPercent = dinReadFloat()
        self.count = dinReadInt()
        self.queueLength = dinReadFloat()




import time
import whatap.pack.tagcount_pack as tagcount_pack
import whatap.osinfo as osinfo
import whatap.agent.secure.security_master as secure
from whatap.util.hash_util import HashUtil as hashutil
from whatap.value.list_value import ListValue
from whatap.value.null_value import NullValue

class TaskInfraProcessTopNAndGroup:
    PROCESS_TOPN = "infra_process_topn"

    PROCESS_TOPN_COMMAND = "cmd"
    PROCESS_TOPN_OWNER = "owner"
    PROCESS_TOPN_CMDLINE = "args"
    PROCESS_TOPN_STIME = "stime"
    PROCESS_TOPN_PID = "pid"
    PROCESS_TOPN_PPID = "ppid"
    PROCESS_TOPN_CPU = "cpu"
    PROCESS_TOPN_RSS = "rss"
    PROCESS_TOPN_VSZ = "vsz"
    PROCESS_TOPN_TOTAL_CHAR_IO = "totalCharIo"
    PROCESS_TOPN_THREAD_COUNT = "threadCnt"
    PROCESS_TOPN_PROCESSINFO = "procInfo"

    PROCESS_GROUP = "infra_process_group"
    PROCESS_GROUP_COMMAND = "cmd"
    PROCESS_GROUP_OWNER = "owner"
    PROCESS_GROUP_CPU = "cpu"
    PROCESS_GROUP_RSS = "rss"
    PROCESS_GROUP_VSZ = "vsz"
    PROCESS_GROUP_TOTAL_CHAR_IO = "totalCharIo"
    PROCESS_GROUP_THREAD_COUNT = "threadCnt"
    PROCESS_GROUP_PROCESS_COUNT = "procCnt"
    PROCESS_GROUP_NAME = "name"


    def __init__(self, interval = 5, topn = 40):
        self.intervalValue = interval
        self.recType = 2
        self.topn = topn

    def interval(self):
        return self.intervalValue

    def process(self, os_, pcode, oid, now):
        (topn, pg) = osinfo.GetTopNAndGroupProc(self.topn)
        topnPack = None
        pgPack = None

        mem_total = osinfo.GetMemorySize()
        if not mem_total or mem_total <= 0:
            mem_total = None

        secu = secure.GetSecurityMaster()
        if topn:
            topnPack = tagcount_pack.TagCountPack()
            topnPack.pcode = pcode
            topnPack.oid = oid
            topnPack.time = now
            topnPack.category = TaskInfraProcessTopNAndGroup.PROCESS_TOPN
            topnPack.putTag("oname", str(secu.oname))
            topnPack.putTag("!rectype", str(self.recType))

        
            idList = ListValue()
            commandList = ListValue()
            ownerList = ListValue()
            cmdlineList = ListValue()
            stimeList = ListValue()
            pidList = ListValue()
            ppidList = ListValue()
            cpuList = ListValue()
            cpuUserList = ListValue()
            cpuSysList = ListValue()
            rssList = ListValue()
            vszList = ListValue()
            totalCharIoList = ListValue()
            threadCountList = ListValue()
            processInfoList = ListValue()
            pRssList = ListValue()

            for proc in topn:
                key = hashutil.hashFromString("{}{}{}{}{}{}".format(proc.cmd1, proc.user, proc.cmd2, proc.createTime, proc.pid, proc.ppid))
                idList.addLong(key)
                commandList.addString(proc.cmd1)
                ownerList.addString(proc.user)
                cmdlineList.addString(proc.cmd2)
                stimeList.addLong(proc.createTime)
                pidList.addLong(proc.pid)
                ppidList.addLong(proc.ppid)
                cpuList.addFloat(proc.cpu)
                cpuUserList.addFloat(getattr(proc, 'cpuUser', 0.0))
                cpuSysList.addFloat(getattr(proc, 'cpuSys', 0.0))
                rssList.addLong(proc.memoryBytes)
                vszList.addLong(proc.vsz)
                totalCharIoList.addFloat(proc.totalCharIO)
                threadCountList.addLong(proc.threadCount)
                processInfoList.addString("{}, {}, {}, {}, {} ({})".format(proc.user, proc.pid, proc.ppid, proc.createTime, proc.cmd1, proc.cmd2))
                if mem_total:
                    pRssList.addFloat(float(proc.memoryBytes) / float(mem_total) * 100.0)
                else:
                    pRssList.addValue(NullValue())

            topnPack.put("@id", idList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_COMMAND, commandList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_OWNER, ownerList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_CMDLINE, cmdlineList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_STIME, stimeList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_PID, pidList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_PPID, ppidList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_CPU, cpuList)
            topnPack.put("cpuUser", cpuUserList)
            topnPack.put("cpuSys", cpuSysList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_RSS, rssList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_VSZ, vszList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_TOTAL_CHAR_IO, totalCharIoList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_THREAD_COUNT, threadCountList)
            topnPack.put(TaskInfraProcessTopNAndGroup.PROCESS_TOPN_PROCESSINFO, processInfoList)
            topnPack.put("pRss", pRssList)

        if pg:
            pgPack = tagcount_pack.TagCountPack()
            pgPack.pcode = pcode
            pgPack.oid = oid
            pgPack.time = now
            pgPack.category = TaskInfraProcessTopNAndGroup.PROCESS_GROUP
            pgPack.putTag("oname", str(secu.oname))
            pgPack.putTag("!rectype", str(self.recType))

            idList = ListValue()
            commandList = ListValue()
            ownerList = ListValue()
            cpuList = ListValue()
            cpuUserList = ListValue()
            cpuSysList = ListValue()
            rssList = ListValue()
            vszList = ListValue()
            totalCharIoList = ListValue()
            threadCountList = ListValue()
            processCountList = ListValue()
            nameList = ListValue()
            pidListVal = ListValue()
            pRssList = ListValue()

            for _, proc in pg.items():
                hash_key = hashutil.hashFromString("{}\\{}\\".format(proc.user, proc.cmd1))
                idList.addLong(hash_key)
                commandList.addString(proc.cmd1)
                ownerList.addString(proc.user)
                cpuList.addFloat(proc.cpu)
                cpuUserList.addFloat(getattr(proc, 'cpuUser', 0.0))
                cpuSysList.addFloat(getattr(proc, 'cpuSys', 0.0))
                rssList.addLong(proc.memoryBytes)
                vszList.addLong(proc.vsz)
                totalCharIoList.addFloat(proc.totalCharIO)
                threadCountList.addLong(proc.threadCount)
                processCountList.addLong(proc.processCount)
                nameList.addString(proc.name if getattr(proc, 'name', None) else proc.cmd1)
                pids = getattr(proc, 'pids', [])
                pidListVal.addString(",".join(str(p) for p in pids))
                if mem_total:
                    avg_rss = float(proc.memoryBytes) / float(proc.processCount) if proc.processCount > 0 else 0.0
                    pRssList.addFloat(avg_rss / float(mem_total) * 100.0)
                else:
                    pRssList.addValue(NullValue())

            pgPack.put("@id", idList)
            pgPack.put(TaskInfraProcessTopNAndGroup.PROCESS_GROUP_COMMAND, commandList)
            pgPack.put(TaskInfraProcessTopNAndGroup.PROCESS_GROUP_OWNER, ownerList)
            pgPack.put(TaskInfraProcessTopNAndGroup.PROCESS_GROUP_CPU, cpuList)
            pgPack.put("cpuUser", cpuUserList)
            pgPack.put("cpuSys", cpuSysList)
            pgPack.put(TaskInfraProcessTopNAndGroup.PROCESS_GROUP_RSS, rssList)
            pgPack.put(TaskInfraProcessTopNAndGroup.PROCESS_GROUP_VSZ, vszList)
            pgPack.put(TaskInfraProcessTopNAndGroup.PROCESS_GROUP_TOTAL_CHAR_IO, totalCharIoList)
            pgPack.put(TaskInfraProcessTopNAndGroup.PROCESS_GROUP_THREAD_COUNT, threadCountList)
            pgPack.put(TaskInfraProcessTopNAndGroup.PROCESS_GROUP_PROCESS_COUNT, processCountList)
            pgPack.put(TaskInfraProcessTopNAndGroup.PROCESS_GROUP_NAME, nameList)
            pgPack.put("pidList", pidListVal)
            pgPack.put("pRss", pRssList)


        return [topnPack, pgPack]





import threading, platform
import whatap.agent.secure.security_master as secure
import whatap.agent.data.data_pack as data
from whatap.agent.counter.task_extrainfo import TaskExtraINFO
from whatap.agent.counter.task_base import TaskBase
from whatap.agent.counter.task_disk import TaskDisk
from whatap.agent.counter.task_network import TaskNet
from whatap.agent.counter.task_proc import TaskProc
from whatap.agent.counter.task_log import TaskLogEvent
from whatap.agent.counter.task_config import TaskConfig
from whatap.agent.counter.task_filesystem import TaskFileSystem
from whatap.agent.counter.task_tcp import TaskTCP
from whatap.agent.counter.task_hostinfo import TaskHostInfo
from whatap.agent.counter.task_infra_cpu import TaskInfraCPU
from whatap.agent.counter.task_infra_mem import TaskInfraMem
from whatap.agent.counter.task_infra_kernel import TaskInfraKernel
from whatap.agent.counter.task_infra_swap import TaskInfraSwap
from whatap.agent.counter.task_infra_disk import TaskInfraDisk
from whatap.agent.counter.task_infra_network import TaskInfraNetwork
from whatap.agent.counter.task_infra_process import TaskInfraProcess
from whatap.agent.counter.task_infra_process_topn_group import TaskInfraProcessTopNAndGroup
from whatap.agent.counter.task_infra_nic_perf import TaskInfraNicPerf
from whatap.agent.counter.task_infra_disk_perf import TaskInfraDiskPerf
from whatap.agent.counter.task_infra_filesystem import TaskInfraFileSystem
from whatap.agent.counter.task_infra_cpu_core import TaskInfraCPUCore
from whatap.agent.counter.task_infra_zfs_perf import TaskInfraZFSPerf
from whatap.agent.counter.task_infra_zpool_perf import TaskInfraZpoolPerf
from whatap.agent.counter.task_infra_netstat import TaskInfraNetstat
from whatap.agent.counter.task_infra_ntp_info import TaskInfraNtpInfo
from whatap.agent.counter.task_infra_nic_info import TaskInfraNicInfo

from whatap.util.date_util import DateUtil as dateutil
import whatap.util.logging_util as logging_util
import whatap.util.panic_util as panic_util
import whatap.osinfo as osinfo
import whatap.agent.conf.configure as config
import whatap.pack.smbase_pack as smbase_pack
import time
from datetime import datetime, timedelta
import multiprocessing

module_loaded = datetime.now()
last_task_timestamp = 0
last_task_name = ""
hangTask = None

AGENT_VERSION = "unknown"  # injected by whatap_infrad at startup (see whatap_infrad:main)

class TimeoutException(Exception):
    pass

class TaskAction(threading.Thread):
    def __init__(self, task, name, lastActTime, timeout, timeoutException):
        threading.Thread.__init__(self)
        self.p = None
        self.os = None
        self.pcode = None
        self.oid = None
        self.now = None
        self.timeout = float(timeout)
        self.done_event = threading.Event()
        self.done_condition = threading.Condition()
        self.hangCheck = False
        self.hangCount = 0

        self.task  = task
        self.name = name
        self.lastActTime = lastActTime

        self.timeoutException = timeoutException
        if timeoutException:
            self.setDaemon(True)

    def set(self, os, pcode, oid, now):
        self.os = os
        self.pcode = pcode
        self.oid = oid
        self.now = now

    def run(self):
        self.done_event.set()
        while True:
            with self.done_condition:
                if self.done_event.is_set():
                    self.done_condition.wait()

            if not self.done_event.is_set():
                try:
                    self.p = self.task.process(self.os, self.pcode, self.oid, self.now)
                except Exception, e:
                    self.p = None
                    logging_util.errorStack(e)

                self.done_event.set()

    def get(self):
        if not self.timeoutException:
            return self.task.process(self.os, self.pcode, self.oid, self.now)

        if self.hangCheck:
            if not self.done_event.is_set():
                self.hangCount += 1
                raise TimeoutException("Hang Exception")
            
        with self.done_condition:
            self.done_event.clear()
            self.done_condition.notify()

        self.done_event.wait(timeout=self.timeout)

        if not self.done_event.is_set():
            self.hangCheck = True
            self.hangCount = 1
            raise TimeoutException("Timeout Exception")
        else:
            self.hangCheck = False
            self.hangCount = 0
            p = self.p
            self.p = None
            return p
        
        return None



class __CounterManager(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self, name='__CounterManager')
        self.setDaemon(True)

    def run(self):
        global last_task_timestamp
        global hangTask

        tasks = []
        securetasks = []
        packListTasks = []

        secu = secure.GetSecurityMaster()
        os = 0
        system = platform.system().lower()
        if system == "windows":
            os = smbase_pack.OS_WINDOW
        elif system == "linux":
            os = smbase_pack.OS_LINUX
        elif system == "darwin":
            os = smbase_pack.OS_OSX
        elif system == "sunos":
            os = smbase_pack.OS_SUNOS
        elif system == "aix":
            os = smbase_pack.OS_AIX
        elif system == "hp-ux":
            os = smbase_pack.OS_HPUX
        else:
            return

        taskTimeoutExcept = config.GetConfig().task_timeout_except
        taskTimeout = config.GetConfig().task_timeout
        taskHealthCheckCount = config.GetConfig().task_health_check_count

        infraOption = config.GetConfig().CategoryV2Enabled
        serverOption = config.GetConfig().CategoryV1Enabled

        #init
        logging_util.info("[CounterManager] Init Start")
        osinfo.GetServerInfoV2()
        if config.GetConfig().task_disk and serverOption:
            osinfo.GetDisk()
        logging_util.info("[CounterManager] Init End")

        if serverOption:
            if config.GetConfig().task_base:
                base = TaskAction(TaskBase(), "Base", 0, taskTimeout, taskTimeoutExcept)
                tasks.append(base)

            # with infra fs
            #Since the data sets are similar, it was transferred to infra fs
            #if config.GetConfig().task_disk:
            #    disk = TaskAction(TaskDisk(), "Disk", 0, taskTimeout, taskTimeoutExcept)
            #    tasks.append(disk)
            #    if taskTimeoutExcept and taskTimeout != 0:
            #        disk.start()
            
            #with infra net
            #if config.GetConfig().task_net:
            #    net = TaskAction(TaskNet(), "Net", 0, taskTimeout, taskTimeoutExcept)
            #    tasks.append(net)

            if config.GetConfig().task_proc:
                proc = TaskAction(TaskProc(), "Proc", 0, taskTimeout, taskTimeoutExcept)
                tasks.append(proc)
            
            if config.GetConfig().task_config:
                con = TaskAction(TaskConfig(), "Config", 0, taskTimeout, taskTimeoutExcept)
                tasks.append(con)

            if config.GetConfig().task_filesystem:
                filesystem = TaskAction(TaskFileSystem(), "Filesystem", 0, taskTimeout, taskTimeoutExcept)
                tasks.append(filesystem)

            if config.GetConfig().task_tcpcheck:
                tcpcheck = TaskAction(TaskTCP(), "TCPCheck", 0, taskTimeout, taskTimeoutExcept)
                tasks.append(tcpcheck)

            if config.GetConfig().task_logevent:
                logevent = TaskAction(TaskLogEvent(), "LogEvent", 0, taskTimeout, taskTimeoutExcept)
                securetasks.append(logevent)

            if config.GetConfig().task_osinfo:
                info = TaskAction(TaskExtraINFO(agent_version = AGENT_VERSION), "OSInfo", 0, taskTimeout, taskTimeoutExcept)
                securetasks.append(info)


        if config.GetConfig().HostInfoV2Enabled or infraOption:
            hostinfo = TaskAction(TaskHostInfo(agent_version = AGENT_VERSION), "HostInfo", 0, taskTimeout, taskTimeoutExcept)
            securetasks.append(hostinfo)

        unfoldingInterval = int(config.GetConfig().CategoryV2Interval)
        foldingInterval = int(config.GetConfig().CategoryV2FoldingInterval)

        #with server_disk
        infraFileSystem = TaskAction(TaskInfraFileSystem(interval = foldingInterval, infra=infraOption, server=(config.GetConfig().task_disk and serverOption)), "InfraFileSystemWithDisk", 0, taskTimeout, taskTimeoutExcept)
        packListTasks.append(infraFileSystem)
            
        #with server_network
        infraNicPerf = TaskAction(TaskInfraNicPerf(interval = foldingInterval, infra = infraOption, server=(config.GetConfig().task_net and serverOption)), "InfraNicPerf", 0, taskTimeout, taskTimeoutExcept)
        packListTasks.append(infraNicPerf)

        if infraOption:
            infraCpu = TaskAction(TaskInfraCPU(unfoldingInterval), "InfraCPU", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraCpu)

            infraMem = TaskAction(TaskInfraMem(unfoldingInterval), "InfraMem", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraMem)

            infraKernel = TaskAction(TaskInfraKernel(unfoldingInterval), "InfraKernel", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraKernel)

            infraSwap = TaskAction(TaskInfraSwap(unfoldingInterval), "InfraSwap", 0, taskTimeoutExcept, taskTimeoutExcept)
            tasks.append(infraSwap)

            if config.GetConfig().DiskIOEnabled:
                infraDisk = TaskAction(TaskInfraDisk(unfoldingInterval), "InfraDisk", 0, taskTimeout, taskTimeoutExcept)
                tasks.append(infraDisk)

            infraNetwork = TaskAction(TaskInfraNetwork(unfoldingInterval), "InfraNetwork", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraNetwork)

            infraProcess= TaskAction(TaskInfraProcess(unfoldingInterval), "InfraProcess", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraProcess)

            infraProcessTopNAndGroup = TaskAction(TaskInfraProcessTopNAndGroup(interval = foldingInterval), "InfraProcessTopNAndGroup", 0, taskTimeout, taskTimeoutExcept)
            packListTasks.append(infraProcessTopNAndGroup)


            if config.GetConfig().DiskIOEnabled:
                infraDiskPerf = TaskAction(TaskInfraDiskPerf(foldingInterval), "InfraDiskPerf", 0, taskTimeout, taskTimeoutExcept)
                tasks.append(infraDiskPerf)

            infraCpuCore = TaskAction(TaskInfraCPUCore(foldingInterval), "InfraCpuCore", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraCpuCore)

            infraZfsInfo = TaskAction(TaskInfraZFSPerf(foldingInterval), "InfraZFSInfo", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraZfsInfo)

            infraZpoolPerf = TaskAction(TaskInfraZpoolPerf(foldingInterval), "InfraZpoolPerf", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraZpoolPerf)

            infraNetstat = TaskAction(TaskInfraNetstat(60), "InfraNetstat", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraNetstat)

            infraNtpInfo = TaskAction(TaskInfraNtpInfo(60), "InfraNtpInfo", 0, taskTimeout, taskTimeoutExcept)
            tasks.append(infraNtpInfo)

            infraNicInfo = TaskAction(TaskInfraNicInfo(60), "InfraNicInfo", 0, taskTimeout, taskTimeoutExcept)
            packListTasks.append(infraNicInfo)

        #thread mode
        if taskTimeoutExcept and  taskTimeout != 0:
            for each_task in tasks:
                each_task.start()

            for each_task in packListTasks:
                each_task.start()
                
            for each_task in securetasks:
                each_task.start()

        while True:
            try:
                time.sleep(0.5)
                now = osinfo.GetUptime()
                now_timestamp = dateutil.now()
                for each_task in tasks:
                    if each_task.task.interval() >= 2 and now >= each_task.lastActTime+each_task.task.interval() :
                        each_task.lastActTime = now
                        try:
                            logging_util.debug("[CounterManager] TaskStart : ", each_task.name)
                            last_task_name = each_task.name
                            
                            each_task.set(os, secu.pcode, secu.oid, now_timestamp)
                            p = each_task.get()

                            if p :
                                logging_util.debug("[CounterManager] TaskSuccess : ", each_task.name)
                                data.SendHide(p)
                                last_task_timestamp = datetime.now()
                        except TimeoutException, e:
                            if each_task.hangCount >= taskHealthCheckCount:
                                hangTask = each_task.name
                            message = "[CounterManager] Task : {}, Status : {}".format(each_task.name, e)
                            logging_util.error(message)
                            panic_util.AgentErrorLogEventAsync(message)
                        except Exception, e:
                            logging_util.logStack(e)

                for each_task in packListTasks:
                    if each_task.task.interval() >= 5 and now >= each_task.lastActTime + each_task.task.interval():
                        each_task.lastActTime = now
                        try:
                            logging_util.debug("[CounterManager] TaskStart : ", each_task.name)
                            last_task_name = each_task.name
                            
                            each_task.set(os, secu.pcode, secu.oid, now_timestamp)
                            ps = each_task.get()

                            if ps :
                                logging_util.debug("[CounterManager] TaskSuccess : ", each_task.name)
                                for p in ps:
                                    if p:
                                        data.SendHide(p)
                                last_task_timestamp = datetime.now()

                        except TimeoutException, e:
                            if each_task.hangCount >= taskHealthCheckCount:
                                hangTask = each_task.name
                            message = "[CounterManager] Task : {}, Status : {}".format(each_task.name, e)
                            logging_util.error(message)
                            panic_util.AgentErrorLogEventAsync(message)
                        except Exception, e:
                            logging_util.logStack(e)


                for each_task in securetasks:
                    if each_task.task.interval() >= 5 and now >= each_task.lastActTime + each_task.task.interval():
                        each_task.lastActTime = now
                        try:
                            logging_util.debug("[CounterManager] TaskStart : ", each_task.name)
                            last_task_name = each_task.name

                            each_task.set(os, secu.pcode, secu.oid, now_timestamp)
                            p = each_task.get()

                            if p :
                                logging_util.debug("[CounterManager] TaskSuccess : ", each_task.name)
                                data.SendSecure(p)
                                last_task_timestamp = datetime.now()
                        except TimeoutException, e:
                            if each_task.hangCount >= taskHealthCheckCount:
                                hangTask = each_task.name
                            message = "[CounterManager] Task : {}, Status : {}".format(each_task.name, e)
                            logging_util.error(message)
                            panic_util.AgentErrorLogEventAsync(message)
                        except Exception, e:
                            logging_util.logStack(e)
            except Exception, e:
                logging_util.logStack(e)

            
counterManager =__CounterManager()

def startCounterManager():
    counterManager.start()

def isOK():
    global last_task_timestamp
    global hangTask
    ret = True
    now = datetime.now()
    if now-module_loaded > timedelta(minutes = config.GetConfig().HealthCheckCoolTime):
        if last_task_timestamp and now-last_task_timestamp > timedelta(minutes = 1):
            ret = ret and False
            message = "[CounterManager] Health Check Fail Timeout - Last Task : {}".format(last_task_name)
            logging_util.error(message)
            panic_util.AgentErrorLogEventAsync(message)
        if hangTask:
            ret = ret and False
            message = "[CounterManager] Health Check Fail Task Hang : {}".format(hangTask)
            logging_util.error(message)
            panic_util.AgentErrorLogEventAsync(message)

    return ret



import os
import sys
import imp

from idlelib.TreeWidget import TreeItem
from idlelib.ClassBrowser import ClassBrowser, ModuleBrowserTreeItem
from idlelib.PyShell import PyShellFileList


class PathBrowser(ClassBrowser):

    def __init__(self, flist, _htest=False):
        """
        _htest - bool, change box location when running htest
        """
        self._htest = _htest
        self.init(flist)

    def settitle(self):
        self.top.wm_title("Path Browser")
        self.top.wm_iconname("Path Browser")

    def rootnode(self):
        return PathBrowserTreeItem()

class PathBrowserTreeItem(TreeItem):

    def GetText(self):
        return "sys.path"

    def GetSubList(self):
        sublist = []
        for dir in sys.path:
            item = DirBrowserTreeItem(dir)
            sublist.append(item)
        return sublist

class DirBrowserTreeItem(TreeItem):

    def __init__(self, dir, packages=[]):
        self.dir = dir
        self.packages = packages

    def GetText(self):
        if not self.packages:
            return self.dir
        else:
            return self.packages[-1] + ": package"

    def GetSubList(self):
        try:
            names = os.listdir(self.dir or os.curdir)
        except os.error:
            return []
        packages = []
        for name in names:
            file = os.path.join(self.dir, name)
            if self.ispackagedir(file):
                nn = os.path.normcase(name)
                packages.append((nn, name, file))
        packages.sort()
        sublist = []
        for nn, name, file in packages:
            item = DirBrowserTreeItem(file, self.packages + [name])
            sublist.append(item)
        for nn, name in self.listmodules(names):
            item = ModuleBrowserTreeItem(os.path.join(self.dir, name))
            sublist.append(item)
        return sublist

    def ispackagedir(self, file):
        if not os.path.isdir(file):
            return 0
        init = os.path.join(file, "__init__.py")
        return os.path.exists(init)

    def listmodules(self, allnames):
        modules = {}
        suffixes = imp.get_suffixes()
        sorted = []
        for suff, mode, flag in suffixes:
            i = -len(suff)
            for name in allnames[:]:
                normed_name = os.path.normcase(name)
                if normed_name[i:] == suff:
                    mod_name = name[:i]
                    if mod_name not in modules:
                        modules[mod_name] = None
                        sorted.append((normed_name, name))
                        allnames.remove(name)
        sorted.sort()
        return sorted

def _path_browser(parent):
    flist = PyShellFileList(parent)
    PathBrowser(flist, _htest=True)
    parent.mainloop()

if __name__ == "__main__":
    from unittest import main
    main('idlelib.idle_test.test_pathbrowser', verbosity=2, exit=False)

    from idlelib.idle_test.htest import run
    run(_path_browser)



import whatap.net as net
import whatap.net.sender as sender

def SendSecure(p):
	sender.Send(net.NET_SECURE_CYPHER, p)

def SendHide(p ):
	sender.Send(net.NET_SECURE_HIDE, p)

def SendNorm(p ) :
	sender.Send(0, p)




import time

class Initializer(object):
    def init(self):
        pass  # Your implementation for Init goes here


def populateAllTags(whatapTags, tags):
    for k, v in tags.iteritems():
        whatapTags.put_string(k, v)


def populate_all_fields(whatapFields, fields):
    for k, v in fields.iteritems():
        whatapFields.put(k, toValue(v))


def toValue(src):
    if isinstance(src, int):
        return src
    elif isinstance(src, int):
        return int(src)
    elif isinstance(src, int):
        return int(src)
    elif isinstance(src, int):
        return int(src)
    elif isinstance(src, float):
        return float(src)
    elif isinstance(src, float):
        return float(src)
    elif isinstance(src, str):
        return src
    else:
        return str(src)





import unittest
from test import test_support

import UserDict, random, string
import gc, weakref


class DictTest(unittest.TestCase):
    def test_constructor(self):
        # calling built-in types without argument must return empty
        self.assertEqual(dict(), {})
        self.assertIsNot(dict(), {})

    def test_literal_constructor(self):
        # check literal constructor for different sized dicts
        # (to exercise the BUILD_MAP oparg).
        for n in (0, 1, 6, 256, 400):
            items = [(''.join(random.sample(string.letters, 8)), i)
                     for i in range(n)]
            random.shuffle(items)
            formatted_items = ('{!r}: {:d}'.format(k, v) for k, v in items)
            dictliteral = '{' + ', '.join(formatted_items) + '}'
            self.assertEqual(eval(dictliteral), dict(items))

    def test_bool(self):
        self.assertIs(not {}, True)
        self.assertTrue({1: 2})
        self.assertIs(bool({}), False)
        self.assertIs(bool({1: 2}), True)

    def test_keys(self):
        d = {}
        self.assertEqual(d.keys(), [])
        d = {'a': 1, 'b': 2}
        k = d.keys()
        self.assertEqual(set(k), {'a', 'b'})
        self.assertIn('a', k)
        self.assertIn('b', k)
        self.assertTrue(d.has_key('a'))
        self.assertTrue(d.has_key('b'))
        self.assertRaises(TypeError, d.keys, None)

    def test_values(self):
        d = {}
        self.assertEqual(d.values(), [])
        d = {1:2}
        self.assertEqual(d.values(), [2])

        self.assertRaises(TypeError, d.values, None)

    def test_items(self):
        d = {}
        self.assertEqual(d.items(), [])

        d = {1:2}
        self.assertEqual(d.items(), [(1, 2)])

        self.assertRaises(TypeError, d.items, None)

    def test_has_key(self):
        d = {}
        self.assertFalse(d.has_key('a'))
        d = {'a': 1, 'b': 2}
        k = d.keys()
        k.sort()
        self.assertEqual(k, ['a', 'b'])

        self.assertRaises(TypeError, d.has_key)

    def test_contains(self):
        d = {}
        self.assertNotIn('a', d)
        self.assertFalse('a' in d)
        self.assertTrue('a' not in d)
        d = {'a': 1, 'b': 2}
        self.assertIn('a', d)
        self.assertIn('b', d)
        self.assertNotIn('c', d)

        self.assertRaises(TypeError, d.__contains__)

    def test_len(self):
        d = {}
        self.assertEqual(len(d), 0)
        d = {'a': 1, 'b': 2}
        self.assertEqual(len(d), 2)

    def test_getitem(self):
        d = {'a': 1, 'b': 2}
        self.assertEqual(d['a'], 1)
        self.assertEqual(d['b'], 2)
        d['c'] = 3
        d['a'] = 4
        self.assertEqual(d['c'], 3)
        self.assertEqual(d['a'], 4)
        del d['b']
        self.assertEqual(d, {'a': 4, 'c': 3})

        self.assertRaises(TypeError, d.__getitem__)

        class BadEq(object):
            def __eq__(self, other):
                raise Exc()
            def __hash__(self):
                return 24

        d = {}
        d[BadEq()] = 42
        self.assertRaises(KeyError, d.__getitem__, 23)

        class Exc(Exception): pass

        class BadHash(object):
            fail = False
            def __hash__(self):
                if self.fail:
                    raise Exc()
                else:
                    return 42

        x = BadHash()
        d[x] = 42
        x.fail = True
        self.assertRaises(Exc, d.__getitem__, x)

    def test_clear(self):
        d = {1:1, 2:2, 3:3}
        d.clear()
        self.assertEqual(d, {})

        self.assertRaises(TypeError, d.clear, None)

    def test_update(self):
        d = {}
        d.update({1:100})
        d.update({2:20})
        d.update({1:1, 2:2, 3:3})
        self.assertEqual(d, {1:1, 2:2, 3:3})

        d.update()
        self.assertEqual(d, {1:1, 2:2, 3:3})

        self.assertRaises((TypeError, AttributeError), d.update, None)

        class SimpleUserDict:
            def __init__(self):
                self.d = {1:1, 2:2, 3:3}
            def keys(self):
                return self.d.keys()
            def __getitem__(self, i):
                return self.d[i]
        d.clear()
        d.update(SimpleUserDict())
        self.assertEqual(d, {1:1, 2:2, 3:3})

        class Exc(Exception): pass

        d.clear()
        class FailingUserDict:
            def keys(self):
                raise Exc
        self.assertRaises(Exc, d.update, FailingUserDict())

        class FailingUserDict:
            def keys(self):
                class BogonIter:
                    def __init__(self):
                        self.i = 1
                    def __iter__(self):
                        return self
                    def next(self):
                        if self.i:
                            self.i = 0
                            return 'a'
                        raise Exc
                return BogonIter()
            def __getitem__(self, key):
                return key
        self.assertRaises(Exc, d.update, FailingUserDict())

        class FailingUserDict:
            def keys(self):
                class BogonIter:
                    def __init__(self):
                        self.i = ord('a')
                    def __iter__(self):
                        return self
                    def next(self):
                        if self.i <= ord('z'):
                            rtn = chr(self.i)
                            self.i += 1
                            return rtn
                        raise StopIteration
                return BogonIter()
            def __getitem__(self, key):
                raise Exc
        self.assertRaises(Exc, d.update, FailingUserDict())

        class badseq(object):
            def __iter__(self):
                return self
            def next(self):
                raise Exc()

        self.assertRaises(Exc, {}.update, badseq())

        self.assertRaises(ValueError, {}.update, [(1, 2, 3)])

    def test_fromkeys(self):
        self.assertEqual(dict.fromkeys('abc'), {'a':None, 'b':None, 'c':None})
        d = {}
        self.assertIsNot(d.fromkeys('abc'), d)
        self.assertEqual(d.fromkeys('abc'), {'a':None, 'b':None, 'c':None})
        self.assertEqual(d.fromkeys((4,5),0), {4:0, 5:0})
        self.assertEqual(d.fromkeys([]), {})
        def g():
            yield 1
        self.assertEqual(d.fromkeys(g()), {1:None})
        self.assertRaises(TypeError, {}.fromkeys, 3)
        class dictlike(dict): pass
        self.assertEqual(dictlike.fromkeys('a'), {'a':None})
        self.assertEqual(dictlike().fromkeys('a'), {'a':None})
        self.assertIsInstance(dictlike.fromkeys('a'), dictlike)
        self.assertIsInstance(dictlike().fromkeys('a'), dictlike)
        class mydict(dict):
            def __new__(cls):
                return UserDict.UserDict()
        ud = mydict.fromkeys('ab')
        self.assertEqual(ud, {'a':None, 'b':None})
        self.assertIsInstance(ud, UserDict.UserDict)
        self.assertRaises(TypeError, dict.fromkeys)

        class Exc(Exception): pass

        class baddict1(dict):
            def __init__(self):
                raise Exc()

        self.assertRaises(Exc, baddict1.fromkeys, [1])

        class BadSeq(object):
            def __iter__(self):
                return self
            def next(self):
                raise Exc()

        self.assertRaises(Exc, dict.fromkeys, BadSeq())

        class baddict2(dict):
            def __setitem__(self, key, value):
                raise Exc()

        self.assertRaises(Exc, baddict2.fromkeys, [1])

        # test fast path for dictionary inputs
        d = dict(zip(range(6), range(6)))
        self.assertEqual(dict.fromkeys(d, 0), dict(zip(range(6), [0]*6)))

        class baddict3(dict):
            def __new__(cls):
                return d
        d = {i : i for i in range(10)}
        res = d.copy()
        res.update(a=None, b=None, c=None)
        self.assertEqual(baddict3.fromkeys({"a", "b", "c"}), res)

    def test_copy(self):
        d = {1:1, 2:2, 3:3}
        self.assertEqual(d.copy(), {1:1, 2:2, 3:3})
        self.assertEqual({}.copy(), {})
        self.assertRaises(TypeError, d.copy, None)

    def test_get(self):
        d = {}
        self.assertIs(d.get('c'), None)
        self.assertEqual(d.get('c', 3), 3)
        d = {'a': 1, 'b': 2}
        self.assertIs(d.get('c'), None)
        self.assertEqual(d.get('c', 3), 3)
        self.assertEqual(d.get('a'), 1)
        self.assertEqual(d.get('a', 3), 1)
        self.assertRaises(TypeError, d.get)
        self.assertRaises(TypeError, d.get, None, None, None)

    def test_setdefault(self):
        # dict.setdefault()
        d = {}
        self.assertIs(d.setdefault('key0'), None)
        d.setdefault('key0', [])
        self.assertIs(d.setdefault('key0'), None)
        d.setdefault('key', []).append(3)
        self.assertEqual(d['key'][0], 3)
        d.setdefault('key', []).append(4)
        self.assertEqual(len(d['key']), 2)
        self.assertRaises(TypeError, d.setdefault)

        class Exc(Exception): pass

        class BadHash(object):
            fail = False
            def __hash__(self):
                if self.fail:
                    raise Exc()
                else:
                    return 42

        x = BadHash()
        d[x] = 42
        x.fail = True
        self.assertRaises(Exc, d.setdefault, x, [])

    def test_setdefault_atomic(self):
        # Issue #13521: setdefault() calls __hash__ and __eq__ only once.
        class Hashed(object):
            def __init__(self):
                self.hash_count = 0
                self.eq_count = 0
            def __hash__(self):
                self.hash_count += 1
                return 42
            def __eq__(self, other):
                self.eq_count += 1
                return id(self) == id(other)
        hashed1 = Hashed()
        y = {hashed1: 5}
        hashed2 = Hashed()
        y.setdefault(hashed2, [])
        self.assertEqual(hashed1.hash_count, 1)
        self.assertEqual(hashed2.hash_count, 1)
        self.assertEqual(hashed1.eq_count + hashed2.eq_count, 1)

    def test_popitem(self):
        # dict.popitem()
        for copymode in -1, +1:
            # -1: b has same structure as a
            # +1: b is a.copy()
            for log2size in range(12):
                size = 2**log2size
                a = {}
                b = {}
                for i in range(size):
                    a[repr(i)] = i
                    if copymode < 0:
                        b[repr(i)] = i
                if copymode > 0:
                    b = a.copy()
                for i in range(size):
                    ka, va = ta = a.popitem()
                    self.assertEqual(va, int(ka))
                    kb, vb = tb = b.popitem()
                    self.assertEqual(vb, int(kb))
                    self.assertFalse(copymode < 0 and ta != tb)
                self.assertFalse(a)
                self.assertFalse(b)

        d = {}
        self.assertRaises(KeyError, d.popitem)

    def test_pop(self):
        # Tests for pop with specified key
        d = {}
        k, v = 'abc', 'def'
        d[k] = v
        self.assertRaises(KeyError, d.pop, 'ghi')

        self.assertEqual(d.pop(k), v)
        self.assertEqual(len(d), 0)

        self.assertRaises(KeyError, d.pop, k)

        # verify longs/ints get same value when key > 32 bits
        # (for 64-bit archs).  See SF bug #689659.
        x = 4503599627370496L
        y = 4503599627370496
        h = {x: 'anything', y: 'something else'}
        self.assertEqual(h[x], h[y])

        self.assertEqual(d.pop(k, v), v)
        d[k] = v
        self.assertEqual(d.pop(k, 1), v)

        self.assertRaises(TypeError, d.pop)

        class Exc(Exception): pass

        class BadHash(object):
            fail = False
            def __hash__(self):
                if self.fail:
                    raise Exc()
                else:
                    return 42

        x = BadHash()
        d[x] = 42
        x.fail = True
        self.assertRaises(Exc, d.pop, x)

    def test_mutatingiteration(self):
        # changing dict size during iteration
        d = {}
        d[1] = 1
        with self.assertRaises(RuntimeError):
            for i in d:
                d[i+1] = 1

    def test_repr(self):
        d = {}
        self.assertEqual(repr(d), '{}')
        d[1] = 2
        self.assertEqual(repr(d), '{1: 2}')
        d = {}
        d[1] = d
        self.assertEqual(repr(d), '{1: {...}}')

        class Exc(Exception): pass

        class BadRepr(object):
            def __repr__(self):
                raise Exc()

        d = {1: BadRepr()}
        self.assertRaises(Exc, repr, d)

    def test_le(self):
        self.assertFalse({} < {})
        self.assertFalse({1: 2} < {1L: 2L})

        class Exc(Exception): pass

        class BadCmp(object):
            def __eq__(self, other):
                raise Exc()
            def __hash__(self):
                return 42

        d1 = {BadCmp(): 1}
        d2 = {1: 1}

        with self.assertRaises(Exc):
            d1 < d2

    def test_missing(self):
        # Make sure dict doesn't have a __missing__ method
        self.assertFalse(hasattr(dict, "__missing__"))
        self.assertFalse(hasattr({}, "__missing__"))
        # Test several cases:
        # (D) subclass defines __missing__ method returning a value
        # (E) subclass defines __missing__ method raising RuntimeError
        # (F) subclass sets __missing__ instance variable (no effect)
        # (G) subclass doesn't define __missing__ at a all
        class D(dict):
            def __missing__(self, key):
                return 42
        d = D({1: 2, 3: 4})
        self.assertEqual(d[1], 2)
        self.assertEqual(d[3], 4)
        self.assertNotIn(2, d)
        self.assertNotIn(2, d.keys())
        self.assertEqual(d[2], 42)

        class E(dict):
            def __missing__(self, key):
                raise RuntimeError(key)
        e = E()
        with self.assertRaises(RuntimeError) as c:
            e[42]
        self.assertEqual(c.exception.args, (42,))

        class F(dict):
            def __init__(self):
                # An instance variable __missing__ should have no effect
                self.__missing__ = lambda key: None
        f = F()
        with self.assertRaises(KeyError) as c:
            f[42]
        self.assertEqual(c.exception.args, (42,))

        class G(dict):
            pass
        g = G()
        with self.assertRaises(KeyError) as c:
            g[42]
        self.assertEqual(c.exception.args, (42,))

    def test_tuple_keyerror(self):
        # SF #1576657
        d = {}
        with self.assertRaises(KeyError) as c:
            d[(1,)]
        self.assertEqual(c.exception.args, ((1,),))

    def test_bad_key(self):
        # Dictionary lookups should fail if __cmp__() raises an exception.
        class CustomException(Exception):
            pass

        class BadDictKey:
            def __hash__(self):
                return hash(self.__class__)

            def __cmp__(self, other):
                if isinstance(other, self.__class__):
                    raise CustomException
                return other

        d = {}
        x1 = BadDictKey()
        x2 = BadDictKey()
        d[x1] = 1
        for stmt in ['d[x2] = 2',
                     'z = d[x2]',
                     'x2 in d',
                     'd.has_key(x2)',
                     'd.get(x2)',
                     'd.setdefault(x2, 42)',
                     'd.pop(x2)',
                     'd.update({x2: 2})']:
            with self.assertRaises(CustomException):
                exec stmt in locals()

    def test_resize1(self):
        # Dict resizing bug, found by Jack Jansen in 2.2 CVS development.
        # This version got an assert failure in debug build, infinite loop in
        # release build.  Unfortunately, provoking this kind of stuff requires
        # a mix of inserts and deletes hitting exactly the right hash codes in
        # exactly the right order, and I can't think of a randomized approach
        # that would be *likely* to hit a failing case in reasonable time.

        d = {}
        for i in range(5):
            d[i] = i
        for i in range(5):
            del d[i]
        for i in range(5, 9):  # i==8 was the problem
            d[i] = i

    def test_resize2(self):
        # Another dict resizing bug (SF bug #1456209).
        # This caused Segmentation faults or Illegal instructions.

        class X(object):
            def __hash__(self):
                return 5
            def __eq__(self, other):
                if resizing:
                    d.clear()
                return False
        d = {}
        resizing = False
        d[X()] = 1
        d[X()] = 2
        d[X()] = 3
        d[X()] = 4
        d[X()] = 5
        # now trigger a resize
        resizing = True
        d[9] = 6

    def test_empty_presized_dict_in_freelist(self):
        # Bug #3537: if an empty but presized dict with a size larger
        # than 7 was in the freelist, it triggered an assertion failure
        with self.assertRaises(ZeroDivisionError):
            d = {'a': 1 // 0, 'b': None, 'c': None, 'd': None, 'e': None,
                 'f': None, 'g': None, 'h': None}
        d = {}

    def test_container_iterator(self):
        # Bug #3680: tp_traverse was not implemented for dictiter objects
        class C(object):
            pass
        iterators = (dict.iteritems, dict.itervalues, dict.iterkeys)
        for i in iterators:
            obj = C()
            ref = weakref.ref(obj)
            container = {obj: 1}
            obj.x = i(container)
            del obj, container
            gc.collect()
            self.assertIs(ref(), None, "Cycle was not collected")

    def _not_tracked(self, t):
        # Nested containers can take several collections to untrack
        gc.collect()
        gc.collect()
        self.assertFalse(gc.is_tracked(t), t)

    def _tracked(self, t):
        self.assertTrue(gc.is_tracked(t), t)
        gc.collect()
        gc.collect()
        self.assertTrue(gc.is_tracked(t), t)

    @test_support.cpython_only
    def test_track_literals(self):
        # Test GC-optimization of dict literals
        x, y, z, w = 1.5, "a", (1, None), []

        self._not_tracked({})
        self._not_tracked({x:(), y:x, z:1})
        self._not_tracked({1: "a", "b": 2})
        self._not_tracked({1: 2, (None, True, False, ()): int})
        self._not_tracked({1: object()})

        # Dicts with mutable elements are always tracked, even if those
        # elements are not tracked right now.
        self._tracked({1: []})
        self._tracked({1: ([],)})
        self._tracked({1: {}})
        self._tracked({1: set()})

    @test_support.cpython_only
    def test_track_dynamic(self):
        # Test GC-optimization of dynamically-created dicts
        class MyObject(object):
            pass
        x, y, z, w, o = 1.5, "a", (1, object()), [], MyObject()

        d = dict()
        self._not_tracked(d)
        d[1] = "a"
        self._not_tracked(d)
        d[y] = 2
        self._not_tracked(d)
        d[z] = 3
        self._not_tracked(d)
        self._not_tracked(d.copy())
        d[4] = w
        self._tracked(d)
        self._tracked(d.copy())
        d[4] = None
        self._not_tracked(d)
        self._not_tracked(d.copy())

        # dd isn't tracked right now, but it may mutate and therefore d
        # which contains it must be tracked.
        d = dict()
        dd = dict()
        d[1] = dd
        self._not_tracked(dd)
        self._tracked(d)
        dd[1] = d
        self._tracked(dd)

        d = dict.fromkeys([x, y, z])
        self._not_tracked(d)
        dd = dict()
        dd.update(d)
        self._not_tracked(dd)
        d = dict.fromkeys([x, y, z, o])
        self._tracked(d)
        dd = dict()
        dd.update(d)
        self._tracked(dd)

        d = dict(x=x, y=y, z=z)
        self._not_tracked(d)
        d = dict(x=x, y=y, z=z, w=w)
        self._tracked(d)
        d = dict()
        d.update(x=x, y=y, z=z)
        self._not_tracked(d)
        d.update(w=w)
        self._tracked(d)

        d = dict([(x, y), (z, 1)])
        self._not_tracked(d)
        d = dict([(x, y), (z, w)])
        self._tracked(d)
        d = dict()
        d.update([(x, y), (z, 1)])
        self._not_tracked(d)
        d.update([(x, y), (z, w)])
        self._tracked(d)

    @test_support.cpython_only
    def test_track_subtypes(self):
        # Dict subtypes are always tracked
        class MyDict(dict):
            pass
        self._tracked(MyDict())


from test import mapping_tests

class GeneralMappingTests(mapping_tests.BasicTestMappingProtocol):
    type2test = dict

class Dict(dict):
    pass

class SubclassMappingTests(mapping_tests.BasicTestMappingProtocol):
    type2test = Dict

def test_main():
    with test_support.check_py3k_warnings(
        ('dict(.has_key..| inequality comparisons) not supported in 3.x',
         DeprecationWarning)):
        test_support.run_unittest(
            DictTest,
            GeneralMappingTests,
            SubclassMappingTests,
        )

if __name__ == "__main__":
    test_main()



import unittest
import pickle
from ctypes import *
import _ctypes_test
dll = CDLL(_ctypes_test.__file__)

class X(Structure):
    _fields_ = [("a", c_int), ("b", c_double)]
    init_called = 0
    def __init__(self, *args, **kw):
        X.init_called += 1
        self.x = 42

class Y(X):
    _fields_ = [("str", c_char_p)]

class PickleTest(unittest.TestCase):
    def dumps(self, item):
        return pickle.dumps(item)

    def loads(self, item):
        return pickle.loads(item)

    def test_simple(self):
        for src in [
            c_int(42),
            c_double(3.14),
            ]:
            dst = self.loads(self.dumps(src))
            self.assertEqual(src.__dict__, dst.__dict__)
            self.assertEqual(memoryview(src).tobytes(),
                                 memoryview(dst).tobytes())

    def test_struct(self):
        X.init_called = 0

        x = X()
        x.a = 42
        self.assertEqual(X.init_called, 1)

        y = self.loads(self.dumps(x))

        # loads must NOT call __init__
        self.assertEqual(X.init_called, 1)

        # ctypes instances are identical when the instance __dict__
        # and the memory buffer are identical
        self.assertEqual(y.__dict__, x.__dict__)
        self.assertEqual(memoryview(y).tobytes(),
                             memoryview(x).tobytes())

    def test_unpickable(self):
        # ctypes objects that are pointers or contain pointers are
        # unpickable.
        self.assertRaises(ValueError, lambda: self.dumps(Y()))

        prototype = CFUNCTYPE(c_int)

        for item in [
            c_char_p(),
            c_wchar_p(),
            c_void_p(),
            pointer(c_int(42)),
            dll._testfunc_p_p,
            prototype(lambda: 42),
            ]:
            self.assertRaises(ValueError, lambda: self.dumps(item))

    def test_wchar(self):
        pickle.dumps(c_char("x"))
        # Issue 5049
        pickle.dumps(c_wchar(u"x"))

class PickleTest_1(PickleTest):
    def dumps(self, item):
        return pickle.dumps(item, 1)

class PickleTest_2(PickleTest):
    def dumps(self, item):
        return pickle.dumps(item, 2)

if __name__ == "__main__":
    unittest.main()



import time
import whatap.pack.tagcount_pack as tagcount_pack 
import whatap.osinfo as osinfo
import whatap.agent.secure.security_master as secure
from whatap.util.hash_util import HashUtil as hashutil
from whatap.value.list_value import ListValue 

class TaskInfraDiskPerf:
    DISK = "infra_disk_perf"

    DISK_NAME = "name"
    DISK_BUSY = "busy"
    DISK_TOTAL_BPS = "total"
    DISK_READ_BPS = "read"
    DISK_WRITE_BPS = "write"
    DISK_TOTAL_IOPS = "totalIops"
    DISK_READ_IOPS = "readIops"
    DISK_WRITE_IOPS = "writeIops"
    DISK_READ_SERVICETIME = "readSvcTime"
    DISK_WRITE_SERVICETIME = "writeSvcTime"
    DISK_AVG_SERVICETIME = "avgSvcTime"
    DISK_AVG_WAITTIME = "avgWaitTime"

    def __init__(self, interval = 5):
        self.intervalValue = interval
        self.recType = 2

    def interval(self):
        return self.intervalValue


    def process(self, os_, pcode, oid, now):
        disk = osinfo.GetDiskIO()
        if not disk:
            return None

        secu = secure.GetSecurityMaster()
        p = tagcount_pack.TagCountPack()
        p.pcode = pcode
        p.oid = oid
        p.time = now
        p.category = TaskInfraDiskPerf.DISK
        p.putTag("oname", str(secu.oname))
        p.putTag("!rectype", str(self.recType))

        idList = ListValue()
        nameList = ListValue()
        busyList = ListValue()
        totalList = ListValue()
        readList = ListValue()
        writeList = ListValue()
        iopsList = ListValue()
        readIopsList = ListValue()
        writeIopsList = ListValue()
        readServiceTimeList = ListValue()
        writeServiceTimeList = ListValue()
        avgServiceTimeList = ListValue()
        avgWaitTimeList = ListValue()

        for d in disk:
            key = hashutil.hashFromString(d.deviceId)
            idList.addLong(key)
            nameList.addString(d.deviceId)
            busyList.addFloat(d.ioPercent)
            if d.totalBps == 0:
                totalList.addFloat(d.readBps + d.writeBps)
            else:
                totalList.addFloat(d.totalBps)
            readList.addFloat(d.readBps)
            writeList.addFloat(d.writeBps)
            if d.totalIops == 0:
                iopsList.addFloat(d.readIops + d.writeIops)
            else:
                iopsList.addFloat(d.totalIops)
            readIopsList.addFloat(d.readIops)
            writeIopsList.addFloat(d.writeIops)
            
            if d.readTime != osinfo._DiskPerf.UNSUPPORTED:
                readServiceTimeList.addFloat(d.readTime)
            if d.writeTime != osinfo._DiskPerf.UNSUPPORTED:
                writeServiceTimeList.addFloat(d.writeTime)
            if d.totalBusyTime != osinfo._DiskPerf.UNSUPPORTED:
                avgServiceTimeList.addFloat(d.totalBusyTime)
            if d.totalWaitTime != osinfo._DiskPerf.UNSUPPORTED:
                avgWaitTimeList.addFloat(d.totalWaitTime)
                


        p.put("@id", idList)
        p.put(TaskInfraDiskPerf.DISK_NAME, nameList)
        p.put(TaskInfraDiskPerf.DISK_BUSY, busyList)
        p.put(TaskInfraDiskPerf.DISK_TOTAL_BPS, totalList)
        p.put(TaskInfraDiskPerf.DISK_READ_BPS, readList)
        p.put(TaskInfraDiskPerf.DISK_WRITE_BPS, writeList)
        p.put(TaskInfraDiskPerf.DISK_TOTAL_IOPS, iopsList)
        p.put(TaskInfraDiskPerf.DISK_READ_IOPS, readIopsList)
        p.put(TaskInfraDiskPerf.DISK_WRITE_IOPS, writeIopsList)
        if readServiceTimeList.size() > 0:
            p.put(TaskInfraDiskPerf.DISK_READ_SERVICETIME, readServiceTimeList)
        if writeServiceTimeList.size() > 0:
            p.put(TaskInfraDiskPerf.DISK_WRITE_SERVICETIME, writeServiceTimeList)
        if avgServiceTimeList.size() > 0:
            p.put(TaskInfraDiskPerf.DISK_AVG_SERVICETIME, avgServiceTimeList)
        if avgWaitTimeList.size() > 0:
            p.put(TaskInfraDiskPerf.DISK_AVG_WAITTIME, avgWaitTimeList)


        return p

